@@ -22,12 +22,42 @@ IDP_USERNAME=${IDP_USERNAME:-myself}
 IDP_PASSWORD=${IDP_PASSWORD:-myself}
 IDP_REMOTE_ID=${IDP_REMOTE_ID:-https://idp.testshib.org/idp/shibboleth}
 IDP_ECP_URL=${IDP_ECP_URL:-https://idp.testshib.org/idp/profile/SAML2/SOAP/ECP}
+IDP_METADATA_URL=${IDP_METADATA_URL:-http://www.testshib.org/metadata/testshib-providers.xml}
 
 MAPPING_REMOTE_TYPE=${MAPPING_REMOTE_TYPE:-eppn}
 MAPPING_USER_NAME=${MAPPING_USER_NAME:-"{0}"}
 
 PROTOCOL_ID=${PROTOCOL_ID:-mapped}
 
+# File paths
+FEDERATION_FILES="$KEYSTONE_PLUGIN/files/federation"
+SHIBBOLETH_XML="/etc/shibboleth/shibboleth2.xml"
+ATTRIBUTE_MAP="/etc/shibboleth/attribute-map.xml"
+
+function configure_apache {
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        local keystone_apache_conf=$(apache_site_config_for keystone-wsgi-public)
+
+        echo "ProxyPass /Shibboleth.sso !" | sudo tee -a $keystone_apache_conf
+
+    else
+        local keystone_apache_conf=$(apache_site_config_for keystone)
+
+        # Add WSGIScriptAlias directive to vhost configuration for port 5000
+        sudo sed -i -e "
+            /<VirtualHost \*:5000>/r $KEYSTONE_PLUGIN/files/federation/shib_apache_alias.txt
+        " $keystone_apache_conf
+    fi
+
+    # Append to the keystone.conf vhost file a <Location> directive for the Shibboleth module
+    # and a <Location> directive for the identity provider
+    cat $KEYSTONE_PLUGIN/files/federation/shib_apache_handler.txt | sudo tee -a $keystone_apache_conf
+
+    sudo sed -i -e "s|%IDP_ID%|$IDP_ID|g;" $keystone_apache_conf
+
+    restart_apache_server
+}
+
 function install_federation {
     if is_ubuntu; then
         install_package libapache2-mod-shib2
@@ -58,22 +88,18 @@ function upload_sp_metadata {
 }
 
 function configure_federation {
-    local keystone_apache_conf=$(apache_site_config_for keystone)
-
-    # Add WSGIScriptAlias directive to vhost configuration for port 5000
-    sudo sed -i -e "
-        /<VirtualHost \*:5000>/r $KEYSTONE_PLUGIN/files/federation/shib_apache_alias.txt
-    " $keystone_apache_conf
-
-    # Append to the keystone.conf vhost file a <Location> directive for the Shibboleth module
-    # and a <Location> directive for the identity provider
-    cat $KEYSTONE_PLUGIN/files/federation/shib_apache_handler.txt | sudo tee -a $keystone_apache_conf
-    sudo sed -i -e "s|%IDP_ID%|$IDP_ID|g;" $keystone_apache_conf
+    configure_apache
 
     # Copy a templated /etc/shibboleth/shibboleth2.xml file...
-    sudo cp $KEYSTONE_PLUGIN/files/federation/shibboleth2.xml /etc/shibboleth/shibboleth2.xml
-    # ... and replace the %HOST_IP% placeholder with the host ip
-    sudo sed -i -e "s|%HOST_IP%|$HOST_IP|g;" /etc/shibboleth/shibboleth2.xml
+    sudo cp $FEDERATION_FILES/shibboleth2.xml $SHIBBOLETH_XML
+    # ... and replace the %HOST_IP%, %IDP_REMOTE_ID%,and %IDP_METADATA_URL% placeholders
+    sudo sed -i -e "
+        s|%HOST_IP%|$HOST_IP|g;
+        s|%IDP_REMOTE_ID%|$IDP_REMOTE_ID|g;
+        s|%IDP_METADATA_URL%|$IDP_METADATA_URL|g;
+        " $SHIBBOLETH_XML
+
+    sudo cp "$FEDERATION_FILES/attribute-map.xml" $ATTRIBUTE_MAP
 
     restart_service shibd
 
@@ -83,6 +109,10 @@ function configure_federation {
     # Specify the header that contains information about the identity provider
     iniset $KEYSTONE_CONF mapped remote_id_attribute "Shib-Identity-Provider"
 
+    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+        restart_service "devstack@keystone"
+    fi
+
     # Register the service provider
     upload_sp_metadata
 }
