@@ -40,14 +40,12 @@ class AssignmentType(object):
         raise exception.AssignmentTypeCalculationError(**locals())
 
 
-class Assignment(base.AssignmentDriverV9):
+class Assignment(base.AssignmentDriverBase):
 
+    @classmethod
     def default_role_driver(self):
         return 'sql'
 
-    def default_resource_driver(self):
-        return 'sql'
-
     def create_grant(self, role_id, user_id=None, group_id=None,
                      domain_id=None, project_id=None,
                      inherited_to_projects=False):
@@ -290,8 +288,70 @@ class Assignment(base.AssignmentDriverV9):
             )
             q.delete(False)
 
+    def create_system_grant(self, role_id, actor_id, target_id,
+                            assignment_type, inherited):
+        try:
+            with sql.session_for_write() as session:
+                session.add(
+                    SystemRoleAssignment(
+                        type=assignment_type,
+                        actor_id=actor_id,
+                        target_id=target_id,
+                        role_id=role_id,
+                        inherited=inherited
+                    )
+                )
+        except sql.DBDuplicateEntry:  # nosec : The v3 grant APIs are silent if
+            # the assignment already exists
+            pass
 
-class RoleAssignment(sql.ModelBase, sql.DictBase):
+    def list_system_grants(self, actor_id, target_id, assignment_type):
+        with sql.session_for_read() as session:
+            query = session.query(SystemRoleAssignment)
+            if actor_id:
+                query = query.filter_by(actor_id=actor_id)
+            if target_id:
+                query = query.filter_by(target_id=target_id)
+            if assignment_type:
+                query = query.filter_by(type=assignment_type)
+            results = query.all()
+
+        return [role.to_dict() for role in results]
+
+    def list_system_grants_by_role(self, role_id):
+        with sql.session_for_read() as session:
+            query = session.query(SystemRoleAssignment)
+            query = query.filter_by(role_id=role_id)
+            return query.all()
+
+    def check_system_grant(self, role_id, actor_id, target_id, inherited):
+        with sql.session_for_read() as session:
+            try:
+                q = session.query(SystemRoleAssignment)
+                q = q.filter_by(actor_id=actor_id)
+                q = q.filter_by(target_id=target_id)
+                q = q.filter_by(role_id=role_id)
+                q = q.filter_by(inherited=inherited)
+                q.one()
+            except sql.NotFound:
+                raise exception.RoleAssignmentNotFound(
+                    role_id=role_id, actor_id=actor_id, target_id=target_id
+                )
+
+    def delete_system_grant(self, role_id, actor_id, target_id, inherited):
+        with sql.session_for_write() as session:
+            q = session.query(SystemRoleAssignment)
+            q = q.filter_by(actor_id=actor_id)
+            q = q.filter_by(target_id=target_id)
+            q = q.filter_by(role_id=role_id)
+            q = q.filter_by(inherited=inherited)
+            if not q.delete(False):
+                raise exception.RoleAssignmentNotFound(
+                    role_id=role_id, actor_id=actor_id, target_id=target_id
+                )
+
+
+class RoleAssignment(sql.ModelBase, sql.ModelDictMixin):
     __tablename__ = 'assignment'
     attributes = ['type', 'actor_id', 'target_id', 'role_id', 'inherited']
     # NOTE(henry-nash): Postgres requires a name to be defined for an Enum
@@ -317,3 +377,26 @@ class RoleAssignment(sql.ModelBase, sql.DictBase):
         parent implementation is not applicable.
         """
         return dict(self.items())
+
+
+class SystemRoleAssignment(sql.ModelBase, sql.ModelDictMixin):
+    __tablename__ = 'system_assignment'
+    attributes = ['type', 'actor_id', 'target_id', 'role_id', 'inherited']
+    type = sql.Column(sql.String(64), nullable=False)
+    actor_id = sql.Column(sql.String(64), nullable=False)
+    target_id = sql.Column(sql.String(64), nullable=False)
+    role_id = sql.Column(sql.String(64), nullable=False)
+    inherited = sql.Column(sql.Boolean, default=False, nullable=False)
+    __table_args__ = (
+        sql.PrimaryKeyConstraint('type', 'actor_id', 'target_id', 'role_id',
+                                 'inherited'),
+        sql.Index('ix_system_actor_id', 'actor_id'),
+    )
+
+    def to_dict(self):
+        """Override parent method with a simpler implementation.
+
+        RoleAssignment doesn't have non-indexed 'extra' attributes, so the
+        parent implementation is not applicable.
+        """
+        return dict(self.items())
