@@ -37,20 +37,6 @@ class Public(wsgi.ComposableRouter):
                        conditions=dict(method=['GET']))
 
 
-class Admin(wsgi.ComposableRouter):
-    def add_routes(self, mapper):
-        # Role Operations
-        roles_controller = controllers.RoleAssignmentV2()
-        mapper.connect('/tenants/{tenant_id}/users/{user_id}/roles',
-                       controller=roles_controller,
-                       action='get_user_roles',
-                       conditions=dict(method=['GET']))
-        mapper.connect('/users/{user_id}/roles',
-                       controller=roles_controller,
-                       action='get_user_roles',
-                       conditions=dict(method=['GET']))
-
-
 class Routers(wsgi.RoutersBase):
 
     def append_v3_routers(self, mapper, routers):
@@ -190,6 +176,44 @@ class Routers(wsgi.RoutersBase):
                 'domain_id': json_home.Parameters.DOMAIN_ID,
                 'group_id': json_home.Parameters.GROUP_ID,
             })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/system/users/{user_id}/roles',
+            get_head_action='list_system_grants_for_user',
+            rel=json_home.build_v3_resource_relation('system_user_roles'),
+            path_vars={
+                'user_id': json_home.Parameters.USER_ID
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/system/users/{user_id}/roles/{role_id}',
+            get_head_action='check_system_grant_for_user',
+            put_action='create_system_grant_for_user',
+            delete_action='revoke_system_grant_for_user',
+            rel=json_home.build_v3_resource_relation('system_user_role'),
+            path_vars={
+                'role_id': json_home.Parameters.ROLE_ID,
+                'user_id': json_home.Parameters.USER_ID
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/system/groups/{group_id}/roles',
+            get_head_action='list_system_grants_for_group',
+            rel=json_home.build_v3_resource_relation('system_group_roles'),
+            path_vars={
+                'group_id': json_home.Parameters.GROUP_ID
+            })
+        self._add_resource(
+            mapper, grant_controller,
+            path='/system/groups/{group_id}/roles/{role_id}',
+            get_head_action='check_system_grant_for_group',
+            put_action='create_system_grant_for_group',
+            delete_action='revoke_system_grant_for_group',
+            rel=json_home.build_v3_resource_relation('system_group_role'),
+            path_vars={
+                'role_id': json_home.Parameters.ROLE_ID,
+                'group_id': json_home.Parameters.GROUP_ID
+            })
 
         self._add_resource(
             mapper, controllers.RoleAssignmentV3(),
