@@ -17,7 +17,7 @@ import six
 
 from keystone.auth.plugins import base
 from keystone.auth.plugins import mapped
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import wsgi
 import keystone.conf
 from keystone import exception
@@ -28,14 +28,14 @@ from keystone.models import token_model
 LOG = log.getLogger(__name__)
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
-@dependency.requires('federation_api', 'identity_api', 'token_provider_api')
 class Token(base.AuthMethodHandler):
 
     def _get_token_ref(self, auth_payload):
         token_id = auth_payload['id']
-        response = self.token_provider_api.validate_token(token_id)
+        response = PROVIDERS.token_provider_api.validate_token(token_id)
         return token_model.KeystoneToken(token_id=token_id,
                                          token_data=response)
 
@@ -44,9 +44,11 @@ class Token(base.AuthMethodHandler):
             raise exception.ValidationError(attribute='id',
                                             target='token')
         token_ref = self._get_token_ref(auth_payload)
-        if token_ref.is_federated_user and self.federation_api:
+        if token_ref.is_federated_user and PROVIDERS.federation_api:
             response_data = mapped.handle_scoped_token(
-                request, token_ref, self.federation_api, self.identity_api)
+                request, token_ref, PROVIDERS.federation_api,
+                PROVIDERS.identity_api
+            )
         else:
             response_data = token_authenticate(request,
                                                token_ref)
@@ -70,6 +72,13 @@ def token_authenticate(request, token_ref):
         # state in Keystone. To do so is to invite elevation of
         # privilege attacks
 
+        project_scoped = 'project' in request.json_body['auth'].get(
+            'scope', {}
+        )
+        domain_scoped = 'domain' in request.json_body['auth'].get(
+            'scope', {}
+        )
+
         if token_ref.oauth_scoped:
             raise exception.ForbiddenAction(
                 action=_(
@@ -80,6 +89,13 @@ def token_authenticate(request, token_ref):
                 action=_(
                     'Using trust-scoped token to create another token. '
                     'Create a new trust-scoped token instead'))
+        elif token_ref.system_scoped and (project_scoped or domain_scoped):
+            raise exception.ForbiddenAction(
+                action=_(
+                    'Using a system-scoped token to create a project-scoped '
+                    'or domain-scoped token is not allowed.'
+                )
+            )
 
         if not CONF.token.allow_rescope_scoped_token:
             # Do not allow conversion from scoped tokens.
