@@ -15,22 +15,22 @@
 
 import itertools
 
-from oslo_config import cfg
 import sqlalchemy
 from sqlalchemy.sql import true
 
-from keystone import catalog
-from keystone.catalog import core
+from keystone.catalog.backends import base
 from keystone.common import driver_hints
 from keystone.common import sql
+from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
-class Region(sql.ModelBase, sql.DictBase):
+class Region(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'region'
     attributes = ['id', 'description', 'parent_region_id']
     id = sql.Column(sql.String(255), primary_key=True)
@@ -49,7 +49,7 @@ class Region(sql.ModelBase, sql.DictBase):
     endpoints = sqlalchemy.orm.relationship("Endpoint", backref="region")
 
 
-class Service(sql.ModelBase, sql.DictBase):
+class Service(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'service'
     attributes = ['id', 'type', 'enabled']
     id = sql.Column(sql.String(64), primary_key=True)
@@ -60,7 +60,7 @@ class Service(sql.ModelBase, sql.DictBase):
     endpoints = sqlalchemy.orm.relationship("Endpoint", backref="service")
 
 
-class Endpoint(sql.ModelBase, sql.DictBase):
+class Endpoint(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'endpoint'
     attributes = ['id', 'interface', 'region_id', 'service_id', 'url',
                   'legacy_endpoint_id', 'enabled']
@@ -80,8 +80,16 @@ class Endpoint(sql.ModelBase, sql.DictBase):
                          server_default=sqlalchemy.sql.expression.true())
     extra = sql.Column(sql.JsonBlob())
 
+    @classmethod
+    def from_dict(cls, endpoint_dict):
+        """Override from_dict to set enabled if missing."""
+        new_dict = endpoint_dict.copy()
+        if new_dict.get('enabled') is None:
+            new_dict['enabled'] = True
+        return super(Endpoint, cls).from_dict(new_dict)
 
-class Catalog(catalog.CatalogDriverV8):
+
+class Catalog(base.CatalogDriverBase):
     # Regions
     def list_regions(self, hints):
         with sql.session_for_read() as session:
@@ -211,11 +219,11 @@ class Catalog(catalog.CatalogDriverV8):
             return ref.to_dict()
 
     # Endpoints
-    def create_endpoint(self, endpoint_id, endpoint_ref):
-        new_endpoint = Endpoint.from_dict(endpoint_ref)
+    def create_endpoint(self, endpoint_id, endpoint):
         with sql.session_for_write() as session:
-            session.add(new_endpoint)
-        return new_endpoint.to_dict()
+            endpoint_ref = Endpoint.from_dict(endpoint)
+            session.add(endpoint_ref)
+            return endpoint_ref.to_dict()
 
     def delete_endpoint(self, endpoint_id):
         with sql.session_for_write() as session:
@@ -251,15 +259,15 @@ class Catalog(catalog.CatalogDriverV8):
             ref.extra = new_endpoint.extra
             return ref.to_dict()
 
-    def get_catalog(self, user_id, tenant_id):
+    def get_catalog(self, user_id, project_id):
         """Retrieve and format the V2 service catalog.
 
         :param user_id: The id of the user who has been authenticated for
             creating service catalog.
-        :param tenant_id: The id of the project. 'tenant_id' will be None
+        :param project_id: The id of the project. 'project_id' will be None
             in the case this being called to create a catalog to go in a
             domain scoped token. In this case, any endpoint that requires
-            a tenant_id as part of their URL will be skipped (as would a whole
+            a project_id as part of their URL will be skipped (as would a whole
             service if, as a consequence, it has no valid endpoints).
 
         :returns: A nested dict representing the service catalog or an
@@ -270,13 +278,13 @@ class Catalog(catalog.CatalogDriverV8):
             itertools.chain(CONF.items(), CONF.eventlet_server.items()))
         substitutions.update({'user_id': user_id})
         silent_keyerror_failures = []
-        if tenant_id:
+        if project_id:
             substitutions.update({
-                'tenant_id': tenant_id,
-                'project_id': tenant_id
+                'tenant_id': project_id,
+                'project_id': project_id
             })
         else:
-            silent_keyerror_failures = ['tenant_id', 'project_id', ]
+            silent_keyerror_failures = ['tenant_id', 'project_id']
 
         with sql.session_for_read() as session:
             endpoints = (session.query(Endpoint).
@@ -289,14 +297,14 @@ class Catalog(catalog.CatalogDriverV8):
                 if not endpoint.service['enabled']:
                     continue
                 try:
-                    formatted_url = core.format_url(
+                    formatted_url = utils.format_url(
                         endpoint['url'], substitutions,
                         silent_keyerror_failures=silent_keyerror_failures)
                     if formatted_url is not None:
                         url = formatted_url
                     else:
                         continue
-                except exception.MalformedEndpoint:
+                except exception.MalformedEndpoint:  # nosec(tkelsey)
                     continue  # this failure is already logged in format_url()
 
                 region = endpoint['region_id']
@@ -313,15 +321,15 @@ class Catalog(catalog.CatalogDriverV8):
 
             return catalog
 
-    def get_v3_catalog(self, user_id, tenant_id):
+    def get_v3_catalog(self, user_id, project_id):
         """Retrieve and format the current V3 service catalog.
 
         :param user_id: The id of the user who has been authenticated for
             creating service catalog.
-        :param tenant_id: The id of the project. 'tenant_id' will be None in
+        :param project_id: The id of the project. 'project_id' will be None in
             the case this being called to create a catalog to go in a domain
             scoped token. In this case, any endpoint that requires a
-            tenant_id as part of their URL will be skipped.
+            project_id as part of their URL will be skipped.
 
         :returns: A list representing the service catalog or an empty list
 
@@ -330,13 +338,13 @@ class Catalog(catalog.CatalogDriverV8):
             itertools.chain(CONF.items(), CONF.eventlet_server.items()))
         d.update({'user_id': user_id})
         silent_keyerror_failures = []
-        if tenant_id:
+        if project_id:
             d.update({
-                'tenant_id': tenant_id,
-                'project_id': tenant_id,
+                'tenant_id': project_id,
+                'project_id': project_id,
             })
         else:
-            silent_keyerror_failures = ['tenant_id', 'project_id', ]
+            silent_keyerror_failures = ['tenant_id', 'project_id']
 
         with sql.session_for_read() as session:
             services = (session.query(Service).filter(
@@ -351,14 +359,14 @@ class Catalog(catalog.CatalogDriverV8):
                     del endpoint['enabled']
                     endpoint['region'] = endpoint['region_id']
                     try:
-                        formatted_url = core.format_url(
+                        formatted_url = utils.format_url(
                             endpoint['url'], d,
                             silent_keyerror_failures=silent_keyerror_failures)
                         if formatted_url:
                             endpoint['url'] = formatted_url
                         else:
                             continue
-                    except exception.MalformedEndpoint:
+                    except exception.MalformedEndpoint:  # nosec(tkelsey)
                         # this failure is already logged in format_url()
                         continue
 
@@ -373,7 +381,52 @@ class Catalog(catalog.CatalogDriverV8):
                 service['name'] = svc.extra.get('name', '')
                 return service
 
-            return [make_v3_service(svc) for svc in services]
+            # Build the unfiltered catalog, this is the catalog that is
+            # returned if endpoint filtering is not performed and the
+            # option of `return_all_endpoints_if_no_filter` is set to true.
+            catalog_ref = [make_v3_service(svc) for svc in services]
+
+            # Filter the `catalog_ref` above by any project-endpoint
+            # association configured by endpoint filter.
+            filtered_endpoints = {}
+            if project_id:
+                filtered_endpoints = (
+                    self.catalog_api.list_endpoints_for_project(project_id))
+            # endpoint filter is enabled, only return the filtered endpoints.
+            if filtered_endpoints:
+                filtered_ids = list(filtered_endpoints.keys())
+                # This is actually working on the copy of `catalog_ref` since
+                # the index will be shifted if remove/add any entry for the
+                # original one.
+                for service in catalog_ref[:]:
+                    endpoints = service['endpoints']
+                    for endpoint in endpoints[:]:
+                        endpoint_id = endpoint['id']
+                        # remove the endpoint that is not associated with
+                        # the project.
+                        if endpoint_id not in filtered_ids:
+                            service['endpoints'].remove(endpoint)
+                            continue
+                        # remove the disabled endpoint from the list.
+                        if not filtered_endpoints[endpoint_id]['enabled']:
+                            service['endpoints'].remove(endpoint)
+                    # NOTE(davechen): The service will not be included in the
+                    # catalog if the service doesn't have any endpoint when
+                    # endpoint filter is enabled, this is inconsistent with
+                    # full catalog that is returned when endpoint filter is
+                    # disabled.
+                    if not service.get('endpoints'):
+                        catalog_ref.remove(service)
+            # When it arrives here it means it's domain scoped token (
+            # `project_id` is not set) or it's a project scoped token
+            # but the endpoint filtering is not performed.
+            # Both of them tell us the endpoint filtering is not enabled, so
+            # check the option of `return_all_endpoints_if_no_filter`, it will
+            # judge whether a full unfiltered catalog or a empty service
+            # catalog will be returned.
+            elif not CONF.endpoint_filter.return_all_endpoints_if_no_filter:
+                return []
+            return catalog_ref
 
     @sql.handle_conflicts(conflict_type='project_endpoint')
     def add_endpoint_to_project(self, endpoint_id, project_id):
