@@ -17,15 +17,15 @@
 import os
 
 import dogpile.cache
+from dogpile.cache import region
 from dogpile.cache import util
 from oslo_cache import core as cache
-from oslo_config import cfg
 
 from keystone.common.cache import _context_cache
+import keystone.conf
 
 
-CONF = cfg.CONF
-CACHE_REGION = cache.create_region()
+CONF = keystone.conf.CONF
 
 
 class RegionInvalidationManager(object):
@@ -53,7 +53,7 @@ class RegionInvalidationManager(object):
         return key == self._region_key
 
 
-class DistributedInvalidationStrategy(object):
+class DistributedInvalidationStrategy(region.RegionInvalidationStrategy):
 
     def __init__(self, region_manager):
         self._region_manager = region_manager
@@ -77,7 +77,7 @@ class DistributedInvalidationStrategy(object):
         return False
 
 
-def key_manger_factory(invalidation_manager, orig_key_mangler):
+def key_mangler_factory(invalidation_manager, orig_key_mangler):
     def key_mangler(key):
         # NOTE(dstanek): Since *all* keys go through the key mangler we
         # need to make sure the region keys don't get the region_id added.
@@ -111,6 +111,8 @@ def create_region(name):
 CACHE_REGION = create_region(name='shared default')
 CACHE_INVALIDATION_REGION = create_region(name='invalidation region')
 
+register_model_handler = _context_cache._register_model_handler
+
 
 def configure_cache(region=None):
     if region is None:
@@ -127,14 +129,10 @@ def configure_cache(region=None):
 
         region_manager = RegionInvalidationManager(
             CACHE_INVALIDATION_REGION, region.name)
-        region.key_mangler = key_manger_factory(
+        region.key_mangler = key_mangler_factory(
             region_manager, region.key_mangler)
-        strategy = DistributedInvalidationStrategy(region_manager)
-
-        def invalidate_by_strategy(*args, **kwargs):
-            return strategy.invalidate()
-
-        region.invalidate = invalidate_by_strategy
+        region.region_invalidator = DistributedInvalidationStrategy(
+            region_manager)
 
 
 def _sha1_mangle_key(key):
@@ -166,6 +164,10 @@ def configure_invalidation_region():
     CACHE_INVALIDATION_REGION.configure_from_config(
         config_dict, '%s.' % CONF.cache.config_prefix)
 
+    # NOTE(breton): Wrap the cache invalidation region to avoid excessive
+    # calls to memcached, which would result in poor performance.
+    CACHE_INVALIDATION_REGION.wrap(_context_cache._ResponseCacheProxy)
+
     # NOTE(morganfainberg): if the backend requests the use of a
     # key_mangler, we should respect that key_mangler function.  If a
     # key_mangler is not defined by the backend, use the sha1_mangle_key
