@@ -11,14 +11,23 @@
 # under the License.
 
 from oslo_config import cfg
+from oslo_log import versionutils
 
 from keystone.conf import constants
 from keystone.conf import utils
 
 
+_DEPRECATED_MSG = utils.fmt("""
+`keystone-manage pki_setup` was deprecated in Mitaka and removed in Pike.
+These options remain for backwards compatibility.
+""")
+
 certfile = cfg.StrOpt(
     'certfile',
     default=constants._CERTFILE,
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 Absolute path to the public certificate file to use for signing responses to
 revocation lists requests. Set this together with `[signing] keyfile`. For
@@ -29,6 +38,9 @@ pki_setup` to generate self-signed certificates.
 keyfile = cfg.StrOpt(
     'keyfile',
     default=constants._KEYFILE,
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 Absolute path to the private key file to use for signing responses to
 revocation lists requests. Set this together with `[signing] certfile`.
@@ -36,6 +48,9 @@ revocation lists requests. Set this together with `[signing] certfile`.
 
 ca_certs = cfg.StrOpt(
     'ca_certs',
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     default='/etc/keystone/ssl/certs/ca.pem',
     help=utils.fmt("""
 Absolute path to the public certificate authority (CA) file to use when
@@ -48,6 +63,9 @@ you are requesting revocation lists in a non-production environment. Use a
 ca_key = cfg.StrOpt(
     'ca_key',
     default='/etc/keystone/ssl/private/cakey.pem',
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 Absolute path to the private certificate authority (CA) key file to use when
 creating self-signed certificates with `keystone-manage pki_setup`. Set this
@@ -60,6 +78,9 @@ key_size = cfg.IntOpt(
     'key_size',
     default=2048,
     min=1024,
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 Key size (in bits) to use when generating a self-signed token signing
 certificate. There is no reason to set this option unless you are requesting
@@ -70,6 +91,9 @@ issued from a trusted certificate authority instead.
 valid_days = cfg.IntOpt(
     'valid_days',
     default=3650,
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 The validity period (in days) to use when generating a self-signed token
 signing certificate. There is no reason to set this option unless you are
@@ -80,6 +104,9 @@ requesting revocation lists in a non-production environment. Use a
 cert_subject = cfg.StrOpt(
     'cert_subject',
     default=('/C=US/ST=Unset/L=Unset/O=Unset/CN=www.example.com'),
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.PIKE,
     help=utils.fmt("""
 The certificate subject to use when generating a self-signed token signing
 certificate. There is no reason to set this option unless you are requesting
