@@ -13,6 +13,7 @@
 import sys
 
 from oslo_config import cfg
+from oslo_log import versionutils
 
 from keystone.conf import utils
 
@@ -29,7 +30,8 @@ enforced according to the `[token] enforce_token_bind` option.
 enforce_token_bind = cfg.StrOpt(
     'enforce_token_bind',
     default='permissive',
-    choices=['disabled', 'permissive', 'strict', 'required'],
+    deprecated_since=versionutils.deprecated.PIKE,
+    deprecated_for_removal=True,
     help=utils.fmt("""
 This controls the token binding enforcement policy on tokens presented to
 keystone with token binding metadata (as specified by the `[token] bind`
@@ -74,13 +76,14 @@ command).
 driver = cfg.StrOpt(
     'driver',
     default='sql',
+    deprecated_since=versionutils.deprecated.PIKE,
+    deprecated_for_removal=True,
     help=utils.fmt("""
 Entry point for the token persistence backend driver in the
-`keystone.token.persistence` namespace. Keystone provides `kvs` and `sql`
-drivers. The `kvs` backend depends on the configuration in the `[kvs]` section.
-The `sql` option (default) depends on the options in your `[database]` section.
-If you're using the `fernet` `[token] provider`, this backend will not be
-utilized to persist tokens at all.
+`keystone.token.persistence` namespace. Keystone provides the `sql`
+driver.  The `sql` option (default) depends on the options in your
+`[database]` section. If you're using the `fernet` `[token] provider`, this
+backend will not be utilized to persist tokens at all.
 """))
 
 caching = cfg.BoolOpt(
@@ -115,7 +118,7 @@ allow_rescope_scoped_token = cfg.BoolOpt(
     'allow_rescope_scoped_token',
     default=True,
     help=utils.fmt("""
-This toggles whether scoped tokens may be be re-scoped to a new project or
+This toggles whether scoped tokens may be re-scoped to a new project or
 domain, thereby preventing users from exchanging a scoped token (including
 those with a default project scope) for any other token. This forces users to
 either authenticate for unscoped tokens (and later exchange that unscoped token
