@@ -17,21 +17,23 @@ import hashlib
 from oslo_serialization import jsonutils
 
 from keystone.common import controller
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import validation
 from keystone.credential import schema
 from keystone import exception
 from keystone.i18n import _
 
 
-@dependency.requires('credential_api')
+PROVIDERS = provider_api.ProviderAPIs
+
+
 class CredentialV3(controller.V3Controller):
     collection_name = 'credentials'
     member_name = 'credential'
 
     def __init__(self):
         super(CredentialV3, self).__init__()
-        self.get_member_from_driver = self.credential_api.get_credential
+        self.get_member_from_driver = PROVIDERS.credential_api.get_credential
 
     def _assign_unique_id(self, ref, trust_id=None):
         # Generates and assigns a unique identifier to
@@ -49,7 +51,8 @@ class CredentialV3(controller.V3Controller):
                 raise exception.ValidationError(attribute='access',
                                                 target='blob')
             ret_ref = ref.copy()
-            ret_ref['id'] = hashlib.sha256(blob['access']).hexdigest()
+            ret_ref['id'] = hashlib.sha256(
+                blob['access'].encode('utf8')).hexdigest()
             # Update the blob with the trust_id, so credentials created
             # with a trust scoped token will result in trust scoped
             # tokens when authentication via ec2tokens happens
@@ -61,13 +64,12 @@ class CredentialV3(controller.V3Controller):
             return super(CredentialV3, self)._assign_unique_id(ref)
 
     @controller.protected()
-    @validation.validated(schema.credential_create, 'credential')
-    def create_credential(self, context, credential):
-        trust_id = self._get_trust_id_for_request(context)
+    def create_credential(self, request, credential):
+        validation.lazy_validate(schema.credential_create, credential)
         ref = self._assign_unique_id(self._normalize_dict(credential),
-                                     trust_id)
-        ref = self.credential_api.create_credential(ref['id'], ref)
-        return CredentialV3.wrap_member(context, ref)
+                                     request.context.trust_id)
+        ref = PROVIDERS.credential_api.create_credential(ref['id'], ref)
+        return CredentialV3.wrap_member(request.context_dict, ref)
 
     @staticmethod
     def _blob_to_json(ref):
@@ -82,27 +84,29 @@ class CredentialV3(controller.V3Controller):
             return ref
 
     @controller.filterprotected('user_id', 'type')
-    def list_credentials(self, context, filters):
-        hints = CredentialV3.build_driver_hints(context, filters)
-        refs = self.credential_api.list_credentials(hints)
+    def list_credentials(self, request, filters):
+        hints = CredentialV3.build_driver_hints(request, filters)
+        refs = PROVIDERS.credential_api.list_credentials(hints)
         ret_refs = [self._blob_to_json(r) for r in refs]
-        return CredentialV3.wrap_collection(context, ret_refs,
+        return CredentialV3.wrap_collection(request.context_dict, ret_refs,
                                             hints=hints)
 
     @controller.protected()
-    def get_credential(self, context, credential_id):
-        ref = self.credential_api.get_credential(credential_id)
+    def get_credential(self, request, credential_id):
+        ref = PROVIDERS.credential_api.get_credential(credential_id)
         ret_ref = self._blob_to_json(ref)
-        return CredentialV3.wrap_member(context, ret_ref)
+        return CredentialV3.wrap_member(request.context_dict, ret_ref)
 
     @controller.protected()
-    @validation.validated(schema.credential_update, 'credential')
-    def update_credential(self, context, credential_id, credential):
+    def update_credential(self, request, credential_id, credential):
+        validation.lazy_validate(schema.credential_update, credential)
         self._require_matching_id(credential_id, credential)
 
-        ref = self.credential_api.update_credential(credential_id, credential)
-        return CredentialV3.wrap_member(context, ref)
+        ref = PROVIDERS.credential_api.update_credential(
+            credential_id, credential
+        )
+        return CredentialV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    def delete_credential(self, context, credential_id):
-        return self.credential_api.delete_credential(credential_id)
+    def delete_credential(self, request, credential_id):
+        return PROVIDERS.credential_api.delete_credential(credential_id)
