@@ -19,10 +19,9 @@ from oslo_log import log
 from oslo_log import versionutils
 import six
 
-from keystone.common import driver_hints
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 from keystone.identity.backends import base
 from keystone.identity.backends.ldap import common as common_ldap
 from keystone.identity.backends.ldap import models
@@ -87,6 +86,11 @@ class Identity(base.IdentityDriverBase):
     def list_users(self, hints):
         return self.user.get_all_filtered(hints)
 
+    def unset_default_project_id(self, project_id):
+        # This function is not implemented for the LDAP backend. The LDAP
+        # backend is readonly.
+        self._disallow_write()
+
     def get_user_by_name(self, user_name, domain_id):
         # domain_id will already have been handled in the Manager layer,
         # parameter left in so this matches the Driver specification
@@ -111,32 +115,37 @@ class Identity(base.IdentityDriverBase):
     def list_groups(self, hints):
         return self.group.get_all_filtered(hints)
 
-    def list_users_in_group(self, group_id, hints):
-        users = []
-        for user_key in self.group.list_group_users(group_id):
+    def _transform_group_member_ids(self, group_member_list):
+        for user_key in group_member_list:
             if self.conf.ldap.group_members_are_ids:
                 user_id = user_key
             else:
                 user_id = self.user._dn_to_id(user_key)
+            yield user_id
 
+    def list_users_in_group(self, group_id, hints):
+        users = []
+        group_members = self.group.list_group_users(group_id)
+        for user_id in self._transform_group_member_ids(group_members):
             try:
                 users.append(self.user.get_filtered(user_id))
             except exception.UserNotFound:
-                msg = ('Group member `%(user_key)s` not found in'
-                       ' `%(group_id)s`. The user should be removed'
-                       ' from the group. The user will be ignored.')
-                LOG.debug(msg, dict(user_key=user_key, group_id=group_id))
+                msg = ('Group member `%(user_id)s` for group `%(group_id)s`'
+                       ' not found in the directory. The user should be'
+                       ' removed from the group. The user will be ignored.')
+                LOG.debug(msg, dict(user_id=user_id, group_id=group_id))
         return users
 
     def check_user_in_group(self, user_id, group_id):
-        user_refs = self.list_users_in_group(group_id, driver_hints.Hints())
-        for x in user_refs:
-            if x['id'] == user_id:
+        # Before doing anything, check that the user exists. This will raise
+        # a not found error if the user doesn't exist so we avoid doing extra
+        # work.
+        self.get_user(user_id)
+        member_list = self.group.list_group_users(group_id)
+        for group_member_id in self._transform_group_member_ids(member_list):
+            if group_member_id == user_id:
                 break
         else:
-            # Try to fetch the user to see if it even exists.  This
-            # will raise a more accurate exception.
-            self.get_user(user_id)
             raise exception.NotFound(_("User '%(user_id)s' not found in"
                                        " group '%(group_id)s'") %
                                      {'user_id': user_id,
@@ -156,8 +165,7 @@ class Identity(base.IdentityDriverBase):
         return self._update_user(user_id, user)
 
     def delete_user(self, user_id):
-        self._disallow_write()
-        self._delete_user(user_id)
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
 
     def change_password(self, user_id, new_password):
         raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
@@ -167,8 +175,7 @@ class Identity(base.IdentityDriverBase):
         self._add_user_to_group(user_id, group_id)
 
     def remove_user_from_group(self, user_id, group_id):
-        self._disallow_write()
-        self._remove_user_from_group(user_id, group_id)
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
 
     def create_group(self, group_id, group):
         self._disallow_write()
@@ -179,8 +186,7 @@ class Identity(base.IdentityDriverBase):
         return self._update_group(group_id, group)
 
     def delete_group(self, group_id):
-        self._disallow_write()
-        return self._delete_group(group_id)
+        raise exception.Forbidden(READ_ONLY_LDAP_ERROR_MESSAGE)
 
     # Test implementations
     def _create_user(self, user_id, user):
@@ -208,26 +214,6 @@ class Identity(base.IdentityDriverBase):
         self.user.update(user_id, user, old_obj)
         return self.user.get_filtered(user_id)
 
-    def _delete_user(self, user_id):
-        msg = _DEPRECATION_MSG % "delete_user"
-        versionutils.report_deprecated_feature(LOG, msg)
-        user = self.user.get(user_id)
-        user_dn = user['dn']
-        groups = self.group.list_user_groups(user_dn)
-        for group in groups:
-            group_ref = self.group.get(group['id'], '*')  # unfiltered
-            group_dn = group_ref['dn']
-            try:
-                super(GroupApi, self.group).remove_member(user_dn, group_dn)
-            except ldap.NO_SUCH_ATTRIBUTE:
-                msg = _LW('User %(user)s was not removed from group %(group)s '
-                          'because the relationship was not found')
-                LOG.warning(msg, {'user': user_id, 'group': group['id']})
-
-        if hasattr(user, 'tenant_id'):
-            self.project.remove_user(user.tenant_id, user_dn)
-        self.user.delete(user_id)
-
     def _create_group(self, group_id, group):
         msg = _DEPRECATION_MSG % "create_group"
         versionutils.report_deprecated_feature(LOG, msg)
@@ -238,11 +224,6 @@ class Identity(base.IdentityDriverBase):
         versionutils.report_deprecated_feature(LOG, msg)
         return common_ldap.filter_entity(self.group.update(group_id, group))
 
-    def _delete_group(self, group_id):
-        msg = _DEPRECATION_MSG % "delete_group"
-        versionutils.report_deprecated_feature(LOG, msg)
-        return self.group.delete(group_id)
-
     def _add_user_to_group(self, user_id, group_id):
         msg = _DEPRECATION_MSG % "add_user_to_group"
         versionutils.report_deprecated_feature(LOG, msg)
@@ -250,13 +231,6 @@ class Identity(base.IdentityDriverBase):
         user_dn = user_ref['dn']
         self.group.add_user(user_dn, group_id, user_id)
 
-    def _remove_user_from_group(self, user_id, group_id):
-        msg = _DEPRECATION_MSG % "remove_user_from_group"
-        versionutils.report_deprecated_feature(LOG, msg)
-        user_ref = self._get_user(user_id)
-        user_dn = user_ref['dn']
-        self.group.remove_user(user_dn, group_id, user_id)
-
 
 # TODO(termie): turn this into a data object and move logic to driver
 class UserApi(common_ldap.EnabledEmuMixIn, common_ldap.BaseLdap):
@@ -409,16 +383,6 @@ class GroupApi(common_ldap.BaseLdap):
             data.pop('description')
         return super(GroupApi, self).create(data)
 
-    def delete(self, group_id):
-        # TODO(spzala): this is only placeholder for group and domain
-        # role support which will be added under bug 1101287
-
-        group_ref = self.get(group_id)
-        group_dn = group_ref['dn']
-        if group_dn:
-            self._delete_tree_nodes(group_dn, ldap.SCOPE_ONELEVEL)
-        super(GroupApi, self).delete(group_id)
-
     def update(self, group_id, values):
         old_obj = self.get(group_id)
         return super(GroupApi, self).update(group_id, values, old_obj)
@@ -433,14 +397,6 @@ class GroupApi(common_ldap.BaseLdap):
                 'User %(user_id)s is already a member of group %(group_id)s') %
                 {'user_id': user_id, 'group_id': group_id})
 
-    def remove_user(self, user_dn, group_id, user_id):
-        group_ref = self.get(group_id)
-        group_dn = group_ref['dn']
-        try:
-            super(GroupApi, self).remove_member(user_dn, group_dn)
-        except ldap.NO_SUCH_ATTRIBUTE:
-            raise exception.UserNotFound(user_id=user_id)
-
     def list_user_groups(self, user_dn):
         """Return a list of groups for which the user is a member."""
         user_dn_esc = ldap.filter.escape_filter_chars(user_dn)
