@@ -18,9 +18,9 @@ from oslo_db import api as oslo_db_api
 import sqlalchemy
 
 from keystone.common import driver_hints
+from keystone.common import password_hashing
 from keystone.common import resource_options
 from keystone.common import sql
-from keystone.common import utils
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
@@ -52,7 +52,7 @@ class Identity(base.IdentityDriverBase):
         https://blueprints.launchpad.net/keystone/+spec/sql-identiy-pam
 
         """
-        return utils.check_password(password, user_ref.password)
+        return password_hashing.check_password(password, user_ref.password)
 
     # Identity interface
     def authenticate(self, user_id, password):
@@ -125,7 +125,6 @@ class Identity(base.IdentityDriverBase):
 
     @sql.handle_conflicts(conflict_type='user')
     def create_user(self, user_id, user):
-        user = utils.hash_user_password(user)
         with sql.session_for_write() as session:
             user_ref = model.User.from_dict(user)
             if self._change_password_required(user_ref):
@@ -170,6 +169,14 @@ class Identity(base.IdentityDriverBase):
             user_refs = sql.filter_limit_query(model.User, query, hints)
             return [base.filter_user(x.to_dict()) for x in user_refs]
 
+    def unset_default_project_id(self, project_id):
+        with sql.session_for_write() as session:
+            query = session.query(model.User)
+            query = query.filter(model.User.default_project_id == project_id)
+
+            for user in query:
+                user.default_project_id = None
+
     def _get_user(self, session, user_id):
         user_ref = session.query(model.User).get(user_id)
         if not user_ref:
@@ -198,7 +205,6 @@ class Identity(base.IdentityDriverBase):
         with sql.session_for_write() as session:
             user_ref = self._get_user(session, user_id)
             old_user_dict = user_ref.to_dict()
-            user = utils.hash_user_password(user)
             for k in user:
                 old_user_dict[k] = user[k]
             new_user = model.User.from_dict(old_user_dict)
@@ -215,8 +221,11 @@ class Identity(base.IdentityDriverBase):
             resource_options.resource_options_ref_to_mapper(
                 user_ref, model.UserOption)
 
-            if 'password' in user and self._change_password_required(user_ref):
-                user_ref.password_ref.expires_at = datetime.datetime.utcnow()
+            if 'password' in user:
+                user_ref.password = user['password']
+                if self._change_password_required(user_ref):
+                    expires_now = datetime.datetime.utcnow()
+                    user_ref.password_ref.expires_at = expires_now
 
             user_ref.extra = new_user.extra
             return base.filter_user(
@@ -230,7 +239,9 @@ class Identity(base.IdentityDriverBase):
         # Validate the new password against the remaining passwords.
         if unique_cnt > 1:
             for password_ref in user_ref.local_user.passwords:
-                if utils.check_password(password, password_ref.password):
+                if password_hashing.check_password(
+                        password,
+                        password_ref.password_hash or password_ref.password):
                     raise exception.PasswordHistoryValidationError(
                         unique_count=unique_cnt)
 
@@ -240,7 +251,7 @@ class Identity(base.IdentityDriverBase):
             if user_ref.password_ref and user_ref.password_ref.self_service:
                 self._validate_minimum_password_age(user_ref)
             self._validate_password_history(new_password, user_ref)
-            user_ref.password = utils.hash_password(new_password)
+            user_ref.password = new_password
             user_ref.password_ref.self_service = True
 
     def _validate_minimum_password_age(self, user_ref):
