@@ -14,6 +14,7 @@
 
 """Main entry point into the Identity service."""
 
+import copy
 import functools
 import itertools
 import operator
@@ -28,9 +29,9 @@ from pycadf import reason
 from keystone import assignment  # TODO(lbragstad): Decouple this dependency
 from keystone.common import cache
 from keystone.common import clean
-from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import manager
+from keystone.common import provider_api
 from keystone.common.validation import validators
 import keystone.conf
 from keystone import exception
@@ -44,6 +45,8 @@ CONF = keystone.conf.CONF
 
 LOG = log.getLogger(__name__)
 
+PROVIDERS = provider_api.ProviderAPIs
+
 MEMOIZE = cache.get_memoization_decorator(group='identity')
 
 ID_MAPPING_REGION = cache.create_region(name='id mapping')
@@ -62,8 +65,7 @@ REGISTRATION_ATTEMPTS = 10
 SQL_DRIVER = 'SQL'
 
 
-@dependency.requires('domain_config_api', 'resource_api')
-class DomainConfigs(dict):
+class DomainConfigs(provider_api.ProviderAPIMixin, dict):
     """Discover, store and provide access to domain specific configs.
 
     The setup_domain_drivers() call will be made via the wrapper from
@@ -176,7 +178,7 @@ class DomainConfigs(dict):
 
             """
             if not new_config['driver'].is_sql:
-                self.domain_config_api.release_registration(domain_id)
+                PROVIDERS.domain_config_api.release_registration(domain_id)
                 return
 
             # To ensure the current domain is the only SQL driver, we attempt
@@ -192,7 +194,7 @@ class DomainConfigs(dict):
 
             domain_registered = 'Unknown'
             for attempt in range(REGISTRATION_ATTEMPTS):
-                if self.domain_config_api.obtain_registration(
+                if PROVIDERS.domain_config_api.obtain_registration(
                         domain_id, SQL_DRIVER):
                     LOG.debug('Domain %s successfully registered to use the '
                               'SQL driver.', domain_id)
@@ -201,7 +203,7 @@ class DomainConfigs(dict):
                 # We failed to register our use, let's find out who is using it
                 try:
                     domain_registered = (
-                        self.domain_config_api.read_registration(
+                        PROVIDERS.domain_config_api.read_registration(
                             SQL_DRIVER))
                 except exception.ConfigRegistrationNotFound:
                     msg = ('While attempting to register domain %(domain)s to '
@@ -226,7 +228,7 @@ class DomainConfigs(dict):
                 # So we don't have it, but someone else does...let's check that
                 # this domain is still valid
                 try:
-                    self.resource_api.get_domain(domain_registered)
+                    PROVIDERS.resource_api.get_domain(domain_registered)
                 except exception.DomainNotFound:
                     msg = ('While attempting to register domain %(domain)s to '
                            'use the SQL driver, found that it was already '
@@ -236,7 +238,7 @@ class DomainConfigs(dict):
                     LOG.debug(msg, {'domain': domain_id,
                                     'old_domain': domain_registered,
                                     'attempt': attempt + 1})
-                    self.domain_config_api.release_registration(
+                    PROVIDERS.domain_config_api.release_registration(
                         domain_registered, type=SQL_DRIVER)
                     continue
 
@@ -289,7 +291,7 @@ class DomainConfigs(dict):
         """
         for domain in resource_api.list_domains():
             domain_config_options = (
-                self.domain_config_api.
+                PROVIDERS.domain_config_api.
                 get_config_with_sensitive_info(domain['id']))
             if domain_config_options:
                 self._load_config_from_database(domain['id'],
@@ -365,7 +367,7 @@ class DomainConfigs(dict):
             return
 
         latest_domain_config = (
-            self.domain_config_api.
+            PROVIDERS.domain_config_api.
             get_config_with_sensitive_info(domain_id))
         domain_config_in_use = domain_id in self
 
@@ -410,7 +412,7 @@ def domains_configured(f):
                 # completed domain config.
                 if not self.domain_configs.configured:
                     self.domain_configs.setup_domain_drivers(
-                        self.driver, self.resource_api)
+                        self.driver, PROVIDERS.resource_api)
         return f(self, *args, **kwargs)
     return wrapper
 
@@ -436,9 +438,6 @@ def exception_translated(exception_type):
 
 
 @notifications.listener
-@dependency.provider('identity_api')
-@dependency.requires('assignment_api', 'credential_api', 'id_mapping_api',
-                     'resource_api', 'shadow_users_api', 'federation_api')
 class Manager(manager.Manager):
     """Default pivot point for the Identity backend.
 
@@ -479,6 +478,7 @@ class Manager(manager.Manager):
     """
 
     driver_namespace = 'keystone.identity'
+    _provides_api = 'identity_api'
 
     _USER = 'user'
     _GROUP = 'group'
@@ -502,6 +502,14 @@ class Manager(manager.Manager):
 
         driver = self._select_identity_driver(domain_id)
 
+        if not driver.is_sql:
+            # The LDAP driver does not support deleting users or groups.
+            # Moreover, we shouldn't destroy users and groups in an unknown
+            # driver. The only time when we should delete users and groups is
+            # when the backend is SQL because the foreign key in the SQL table
+            # forces us to.
+            return
+
         user_refs = self.list_users(domain_scope=domain_id)
         group_refs = self.list_groups(domain_scope=domain_id)
 
@@ -518,10 +526,7 @@ class Manager(manager.Manager):
         # And finally, delete the users themselves
         for user in user_refs:
             try:
-                if not driver.is_sql:
-                    self.shadow_users_api.delete_user(user['id'])
-                else:
-                    self.delete_user(user['id'])
+                self.delete_user(user['id'])
             except exception.UserNotFound:
                 LOG.debug(('User %(userid)s not found when deleting domain '
                            'contents for %(domainid)s, continuing with '
@@ -617,7 +622,7 @@ class Manager(manager.Manager):
         public_id = None
         if driver.generates_uuids():
             public_id = ref['id']
-        ref['id'] = self.id_mapping_api.create_id_mapping(
+        ref['id'] = PROVIDERS.id_mapping_api.create_id_mapping(
             local_entity, public_id)
         LOG.debug('Created new mapping to public ID: %s', ref['id'])
 
@@ -632,7 +637,7 @@ class Manager(manager.Manager):
             local_entity = {'domain_id': ref['domain_id'],
                             'local_id': ref['id'],
                             'entity_type': entity_type}
-            public_id = self.id_mapping_api.get_public_id(local_entity)
+            public_id = PROVIDERS.id_mapping_api.get_public_id(local_entity)
             if public_id:
                 ref['id'] = public_id
                 LOG.debug('Found existing mapping to public ID: %s',
@@ -670,7 +675,7 @@ class Manager(manager.Manager):
 
         # fetch all mappings for the domain, lookup the user at the map built
         # at previous step and replace his id.
-        domain_mappings = self.id_mapping_api.get_domain_mapping_list(
+        domain_mappings = PROVIDERS.id_mapping_api.get_domain_mapping_list(
             domain_id)
         for _mapping in domain_mappings:
             idx = (_mapping.local_id, _mapping.entity_type, _mapping.domain_id)
@@ -784,7 +789,7 @@ class Manager(manager.Manager):
         # assume it needs mapping, so long as we are using domain specific
         # drivers.
         if conf.domain_specific_drivers_enabled:
-            local_id_ref = self.id_mapping_api.get_id_mapping(public_id)
+            local_id_ref = PROVIDERS.id_mapping_api.get_id_mapping(public_id)
             if local_id_ref:
                 return (
                     local_id_ref['domain_id'],
@@ -814,7 +819,7 @@ class Manager(manager.Manager):
         if not CONF.identity_mapping.backward_compatible_ids:
             # We are not running in backward compatibility mode, so we
             # must use a mapping.
-            local_id_ref = self.id_mapping_api.get_id_mapping(public_id)
+            local_id_ref = PROVIDERS.id_mapping_api.get_id_mapping(public_id)
             if local_id_ref:
                 return (
                     local_id_ref['domain_id'],
@@ -916,14 +921,16 @@ class Manager(manager.Manager):
         ref = self._set_domain_id_and_mapping(
             ref, domain_id, driver, mapping.EntityType.USER)
         ref = self._shadow_nonlocal_user(ref)
-        self.shadow_users_api.set_last_active_at(ref['id'])
+        PROVIDERS.shadow_users_api.set_last_active_at(ref['id'])
         return ref
 
     def _assert_default_project_id_is_not_domain(self, default_project_id):
         if default_project_id:
             # make sure project is not a domain
             try:
-                project_ref = self.resource_api.get_project(default_project_id)
+                project_ref = PROVIDERS.resource_api.get_project(
+                    default_project_id
+                )
                 if project_ref['is_domain'] is True:
                     msg = _("User's default project ID cannot be a "
                             "domain ID: %s")
@@ -944,7 +951,7 @@ class Manager(manager.Manager):
         user.setdefault('enabled', True)
         user['enabled'] = clean.user_enabled(user['enabled'])
         domain_id = user['domain_id']
-        self.resource_api.get_domain(domain_id)
+        PROVIDERS.resource_api.get_domain(domain_id)
 
         self._assert_default_project_id_is_not_domain(
             user_ref.get('default_project_id'))
@@ -979,7 +986,7 @@ class Manager(manager.Manager):
         """
         if user is None:
             user = self.get_user(user_id)
-        self.resource_api.assert_domain_enabled(user['domain_id'])
+        PROVIDERS.resource_api.assert_domain_enabled(user['domain_id'])
         if not user.get('enabled', True):
             raise AssertionError(_('User is disabled: %s') % user_id)
 
@@ -1031,12 +1038,23 @@ class Manager(manager.Manager):
                     raise exception.InvalidOperatorError(op)
         return hints
 
-    def _handle_federated_attributes_in_hints(self, driver, hints):
+    def _handle_shadow_and_local_users(self, driver, hints):
         federated_attributes = ['idp_id', 'protocol_id', 'unique_id']
         for filter_ in hints.filters:
             if filter_['name'] in federated_attributes:
-                return self.shadow_users_api.get_federated_users(hints)
-        return driver.list_users(hints)
+                return PROVIDERS.shadow_users_api.get_federated_users(hints)
+        fed_hints = copy.deepcopy(hints)
+        res = driver.list_users(hints)
+
+        # Note: If the filters contain 'name', we should get the user from both
+        # local user and shadow user backend.
+        for filter_ in fed_hints.filters:
+            if filter_['name'] == 'name':
+                fed_res = PROVIDERS.shadow_users_api.get_federated_users(
+                    fed_hints)
+                res += fed_res
+                break
+        return res
 
     @domains_configured
     @exception_translated('user')
@@ -1053,7 +1071,7 @@ class Manager(manager.Manager):
             # driver selection, so remove any such filter.
             self._mark_domain_id_filter_satisfied(hints)
         hints = self._translate_expired_password_hints(hints)
-        ref_list = self._handle_federated_attributes_in_hints(driver, hints)
+        ref_list = self._handle_shadow_and_local_users(driver, hints)
         return self._set_domain_id_and_mapping(
             ref_list, domain_scope, driver, mapping.EntityType.USER)
 
@@ -1123,12 +1141,12 @@ class Manager(manager.Manager):
         # Get user details to invalidate the cache.
         user_old = self.get_user(user_id)
         driver.delete_user(entity_id)
-        self.assignment_api.delete_user_assignments(user_id)
+        PROVIDERS.assignment_api.delete_user_assignments(user_id)
         self.get_user.invalidate(self, user_id)
         self.get_user_by_name.invalidate(self, user_old['name'],
                                          user_old['domain_id'])
-        self.credential_api.delete_credentials_for_user(user_id)
-        self.id_mapping_api.delete_id_mapping(user_id)
+        PROVIDERS.credential_api.delete_credentials_for_user(user_id)
+        PROVIDERS.id_mapping_api.delete_id_mapping(user_id)
         notifications.Audit.deleted(self._USER, user_id, initiator)
 
         # Invalidate user role assignments cache region, as it may be caching
@@ -1141,7 +1159,7 @@ class Manager(manager.Manager):
         group = group_ref.copy()
         group.setdefault('description', '')
         domain_id = group['domain_id']
-        self.resource_api.get_domain(domain_id)
+        PROVIDERS.resource_api.get_domain(domain_id)
 
         # For creating a group, the domain is in the object itself
         domain_id = group_ref['domain_id']
@@ -1198,12 +1216,14 @@ class Manager(manager.Manager):
     def delete_group(self, group_id, initiator=None):
         domain_id, driver, entity_id = (
             self._get_domain_driver_and_entity_id(group_id))
-        roles = self.assignment_api.list_role_assignments(group_id=group_id)
+        roles = PROVIDERS.assignment_api.list_role_assignments(
+            group_id=group_id
+        )
         user_ids = (u['id'] for u in self.list_users_in_group(group_id))
         driver.delete_group(entity_id)
         self.get_group.invalidate(self, group_id)
-        self.id_mapping_api.delete_id_mapping(group_id)
-        self.assignment_api.delete_group_assignments(group_id)
+        PROVIDERS.id_mapping_api.delete_id_mapping(group_id)
+        PROVIDERS.assignment_api.delete_group_assignments(group_id)
 
         notifications.Audit.deleted(self._GROUP, group_id, initiator)
 
@@ -1393,9 +1413,9 @@ class Manager(manager.Manager):
     @MEMOIZE
     def _shadow_nonlocal_user(self, user):
         try:
-            return self.shadow_users_api.get_user(user['id'])
+            return PROVIDERS.shadow_users_api.get_user(user['id'])
         except exception.UserNotFound:
-            return self.shadow_users_api.create_nonlocal_user(user)
+            return PROVIDERS.shadow_users_api.create_nonlocal_user(user)
 
     @MEMOIZE
     def shadow_federated_user(self, idp_id, protocol_id, unique_id,
@@ -1411,12 +1431,12 @@ class Manager(manager.Manager):
         """
         user_dict = {}
         try:
-            self.shadow_users_api.update_federated_user_display_name(
+            PROVIDERS.shadow_users_api.update_federated_user_display_name(
                 idp_id, protocol_id, unique_id, display_name)
-            user_dict = self.shadow_users_api.get_federated_user(
+            user_dict = PROVIDERS.shadow_users_api.get_federated_user(
                 idp_id, protocol_id, unique_id)
         except exception.UserNotFound:
-            idp = self.federation_api.get_idp(idp_id)
+            idp = PROVIDERS.federation_api.get_idp(idp_id)
             federated_dict = {
                 'idp_id': idp_id,
                 'protocol_id': protocol_id,
@@ -1424,17 +1444,19 @@ class Manager(manager.Manager):
                 'display_name': display_name
             }
             user_dict = (
-                self.shadow_users_api.create_federated_user(idp['domain_id'],
-                                                            federated_dict))
-        self.shadow_users_api.set_last_active_at(user_dict['id'])
+                PROVIDERS.shadow_users_api.create_federated_user(
+                    idp['domain_id'], federated_dict
+                )
+            )
+        PROVIDERS.shadow_users_api.set_last_active_at(user_dict['id'])
         return user_dict
 
 
-@dependency.provider('id_mapping_api')
 class MappingManager(manager.Manager):
     """Default pivot point for the ID Mapping backend."""
 
     driver_namespace = 'keystone.identity.id_mapping'
+    _provides_api = 'id_mapping_api'
 
     def __init__(self):
         super(MappingManager, self).__init__(CONF.identity_mapping.driver)
@@ -1482,11 +1504,11 @@ class MappingManager(manager.Manager):
         ID_MAPPING_REGION.invalidate()
 
 
-@dependency.provider('shadow_users_api')
 class ShadowUsersManager(manager.Manager):
     """Default pivot point for the Shadow Users backend."""
 
     driver_namespace = 'keystone.identity.shadow_users'
+    _provides_api = 'shadow_users_api'
 
     def __init__(self):
         shadow_driver = CONF.shadow_users.driver
