@@ -12,9 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.common import dependency
 from keystone.common import sql
-from keystone import identity
+from keystone.identity.mapping_backends import base
 from keystone.identity.mapping_backends import mapping as identity_mapping
 
 
@@ -35,8 +34,7 @@ class IDMapping(sql.ModelBase, sql.ModelDictMixin):
         sql.UniqueConstraint('domain_id', 'local_id', 'entity_type'),)
 
 
-@dependency.requires('id_generator_api')
-class Mapping(identity.MappingDriverV8):
+class Mapping(base.MappingDriverBase):
 
     def get_public_id(self, local_entity):
         # NOTE(henry-nash): Since the Public ID is regeneratable, rather
@@ -57,6 +55,10 @@ class Mapping(identity.MappingDriverV8):
             except sql.NotFound:
                 return None
 
+    def get_domain_mapping_list(self, domain_id):
+        with sql.session_for_read() as session:
+            return session.query(IDMapping).filter_by(domain_id=domain_id)
+
     def get_id_mapping(self, public_id):
         with sql.session_for_read() as session:
             mapping_ref = session.query(IDMapping).get(public_id)
@@ -65,12 +67,17 @@ class Mapping(identity.MappingDriverV8):
 
     def create_id_mapping(self, local_entity, public_id=None):
         entity = local_entity.copy()
-        with sql.session_for_write() as session:
-            if public_id is None:
-                public_id = self.id_generator_api.generate_public_ID(entity)
-            entity['public_id'] = public_id
-            mapping_ref = IDMapping.from_dict(entity)
-            session.add(mapping_ref)
+        try:
+            with sql.session_for_write() as session:
+                if public_id is None:
+                    public_id = self.id_generator_api.generate_public_ID(
+                        entity)
+                entity['public_id'] = public_id
+                mapping_ref = IDMapping.from_dict(entity)
+                session.add(mapping_ref)
+        except sql.DBDuplicateEntry:
+            # something else created the mapping already. We can use it.
+            public_id = self.get_public_id(local_entity)
         return public_id
 
     def delete_id_mapping(self, public_id):
