@@ -12,15 +12,19 @@
 
 from keystone.common import validation
 from keystone.common.validation import parameter_types
+from keystone.identity.backends import resource_options as ro
 
 
 # NOTE(lhcheng): the max length is not applicable since it is specific
 # to the SQL backend, LDAP does not have length limitation.
 _identity_name = {
     'type': 'string',
-    'minLength': 1
+    'minLength': 1,
+    'pattern': '[\S]+'
 }
 
+# Schema for Identity v3 API
+
 _user_properties = {
     'default_project_id': validation.nullable(parameter_types.id_string),
     'description': validation.nullable(parameter_types.description),
@@ -29,13 +33,19 @@ _user_properties = {
     'name': _identity_name,
     'password': {
         'type': ['string', 'null']
-    }
+    },
+    'options': ro.USER_OPTIONS_REGISTRY.json_schema
 }
 
+# TODO(notmorgan): Provide a mechanism for options to supply real jsonschema
+# validation based upon the option object and the option validator(s)
 user_create = {
     'type': 'object',
     'properties': _user_properties,
     'required': ['name'],
+    'options': {
+        'type': 'object'
+    },
     'additionalProperties': True
 }
 
@@ -43,6 +53,9 @@ user_update = {
     'type': 'object',
     'properties': _user_properties,
     'minProperties': 1,
+    'options': {
+        'type': 'object'
+    },
     'additionalProperties': True
 }
 
