@@ -30,7 +30,7 @@ from pycadf import eventfactory
 from pycadf import reason
 from pycadf import resource
 
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
@@ -79,6 +79,7 @@ CONF = keystone.conf.CONF
 INVALIDATE_USER_TOKEN_PERSISTENCE = 'invalidate_user_tokens'
 INVALIDATE_USER_PROJECT_TOKEN_PERSISTENCE = 'invalidate_user_project_tokens'
 INVALIDATE_USER_OAUTH_CONSUMER_TOKENS = 'invalidate_user_consumer_tokens'
+INVALIDATE_TOKEN_CACHE_DELETED_IDP = 'invalidate_token_cache_from_deleted_idp'
 DOMAIN_DELETED = 'domain_deleted'
 
 
@@ -656,8 +657,7 @@ def send_saml_audit_notification(action, request, user_id, group_ids,
     _send_audit_notification(action, initiator, outcome, target, event_type)
 
 
-@dependency.requires('catalog_api')
-class _CatalogHelperObj(object):
+class _CatalogHelperObj(provider_api.ProviderAPIMixin, object):
     """A helper object to allow lookups of identity service id."""
 
 
