@@ -15,12 +15,10 @@
 
 """Workflow Logic the Resource service."""
 
-import uuid
-
 from six.moves import http_client
 
 from keystone.common import controller
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import validation
 from keystone.common import wsgi
 import keystone.conf
@@ -30,109 +28,22 @@ from keystone.resource import schema
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
-@dependency.requires('resource_api')
-class Tenant(controller.V2Controller):
-
-    @controller.v2_deprecated
-    def get_all_projects(self, request, **kw):
-        """Get a list of all tenants for an admin user."""
-        self.assert_admin(request)
-
-        name = request.params.get('name')
-        if name:
-            return self._get_project_by_name(name)
-
-        try:
-            tenant_refs = self.resource_api.list_projects_in_domain(
-                CONF.identity.default_domain_id)
-        except exception.DomainNotFound:
-            # If the default domain doesn't exist then there are no V2
-            # projects.
-            tenant_refs = []
-        tenant_refs = [self.v3_to_v2_project(tenant_ref)
-                       for tenant_ref in tenant_refs
-                       if not tenant_ref.get('is_domain')]
-        params = {
-            'limit': request.params.get('limit'),
-            'marker': request.params.get('marker'),
-        }
-        return self.format_project_list(tenant_refs, **params)
-
-    def _assert_not_is_domain_project(self, project_id, project_ref=None):
-        # Projects acting as a domain should not be visible via v2
-        if not project_ref:
-            project_ref = self.resource_api.get_project(project_id)
-        if project_ref.get('is_domain'):
-            raise exception.ProjectNotFound(project_id)
-
-    @controller.v2_deprecated
-    def get_project(self, request, tenant_id):
-        # TODO(termie): this stuff should probably be moved to middleware
-        self.assert_admin(request)
-        ref = self.resource_api.get_project(tenant_id)
-        self._assert_not_is_domain_project(tenant_id, ref)
-        return {'tenant': self.v3_to_v2_project(ref)}
-
-    def _get_project_by_name(self, tenant_name):
-        # Projects acting as a domain should not be visible via v2
-        ref = self.resource_api.get_project_by_name(
-            tenant_name, CONF.identity.default_domain_id)
-        self._assert_not_is_domain_project(ref['id'], ref)
-        return {'tenant': self.v3_to_v2_project(ref)}
-
-    # CRUD Extension
-    @controller.v2_deprecated
-    def create_project(self, request, tenant):
-        tenant_ref = self._normalize_dict(tenant)
-
-        validation.lazy_validate(schema.tenant_create, tenant)
-        self.assert_admin(request)
-
-        self.resource_api.ensure_default_domain_exists()
-
-        tenant_ref['id'] = tenant_ref.get('id', uuid.uuid4().hex)
-        tenant = self.resource_api.create_project(
-            tenant_ref['id'],
-            self._normalize_domain_id(request, tenant_ref),
-            initiator=request.audit_initiator)
-        return {'tenant': self.v3_to_v2_project(tenant)}
-
-    @controller.v2_deprecated
-    def update_project(self, request, tenant_id, tenant):
-        validation.lazy_validate(schema.tenant_update, tenant)
-        self.assert_admin(request)
-        self._assert_not_is_domain_project(tenant_id)
-
-        tenant_ref = self.resource_api.update_project(
-            tenant_id, tenant, initiator=request.audit_initiator)
-        return {'tenant': self.v3_to_v2_project(tenant_ref)}
-
-    @controller.v2_deprecated
-    def delete_project(self, request, tenant_id):
-        self.assert_admin(request)
-        self._assert_not_is_domain_project(tenant_id)
-        self.resource_api.delete_project(
-            tenant_id,
-            initiator=request.audit_initiator
-        )
-
-
-@dependency.requires('resource_api')
 class DomainV3(controller.V3Controller):
     collection_name = 'domains'
     member_name = 'domain'
 
     def __init__(self):
         super(DomainV3, self).__init__()
-        self.get_member_from_driver = self.resource_api.get_domain
+        self.get_member_from_driver = PROVIDERS.resource_api.get_domain
 
     @controller.protected()
     def create_domain(self, request, domain):
         validation.lazy_validate(schema.domain_create, domain)
         ref = self._assign_unique_id(self._normalize_dict(domain))
-        ref = self.resource_api.create_domain(
+        ref = PROVIDERS.resource_api.create_domain(
             ref['id'], ref, initiator=request.audit_initiator
         )
         return DomainV3.wrap_member(request.context_dict, ref)
@@ -140,42 +51,43 @@ class DomainV3(controller.V3Controller):
     @controller.filterprotected('enabled', 'name')
     def list_domains(self, request, filters):
         hints = DomainV3.build_driver_hints(request, filters)
-        refs = self.resource_api.list_domains(hints=hints)
+        refs = PROVIDERS.resource_api.list_domains(hints=hints)
         return DomainV3.wrap_collection(request.context_dict,
                                         refs, hints=hints)
 
     @controller.protected()
     def get_domain(self, request, domain_id):
-        ref = self.resource_api.get_domain(domain_id)
+        ref = PROVIDERS.resource_api.get_domain(domain_id)
         return DomainV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
     def update_domain(self, request, domain_id, domain):
         validation.lazy_validate(schema.domain_update, domain)
         self._require_matching_id(domain_id, domain)
-        ref = self.resource_api.update_domain(
+        ref = PROVIDERS.resource_api.update_domain(
             domain_id, domain, initiator=request.audit_initiator
         )
         return DomainV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
     def delete_domain(self, request, domain_id):
-        return self.resource_api.delete_domain(
+        return PROVIDERS.resource_api.delete_domain(
             domain_id, initiator=request.audit_initiator
         )
 
 
-@dependency.requires('domain_config_api')
-@dependency.requires('resource_api')
 class DomainConfigV3(controller.V3Controller):
     member_name = 'config'
 
     @controller.protected()
     def create_domain_config(self, request, domain_id, config):
-        self.resource_api.get_domain(domain_id)
+        PROVIDERS.resource_api.get_domain(domain_id)
         original_config = (
-            self.domain_config_api.get_config_with_sensitive_info(domain_id))
-        ref = self.domain_config_api.create_config(domain_id, config)
+            PROVIDERS.domain_config_api.get_config_with_sensitive_info(
+                domain_id
+            )
+        )
+        ref = PROVIDERS.domain_config_api.create_config(domain_id, config)
         if original_config:
             # Return status code 200, since config already existed
             return wsgi.render_response(body={self.member_name: ref})
@@ -199,55 +111,54 @@ class DomainConfigV3(controller.V3Controller):
     @controller.protected()
     def get_security_compliance_domain_config(self, request, domain_id,
                                               group=None, option=None):
-        ref = self.domain_config_api.get_security_compliance_config(
+        ref = PROVIDERS.domain_config_api.get_security_compliance_config(
             domain_id, group, option=option
         )
         return {self.member_name: ref}
 
     @controller.protected()
     def get_domain_config(self, request, domain_id, group=None, option=None):
-        self.resource_api.get_domain(domain_id)
-        ref = self.domain_config_api.get_config(domain_id, group, option)
+        PROVIDERS.resource_api.get_domain(domain_id)
+        ref = PROVIDERS.domain_config_api.get_config(domain_id, group, option)
         return {self.member_name: ref}
 
     @controller.protected()
     def update_domain_config(
             self, request, domain_id, config, group, option):
-        self.resource_api.get_domain(domain_id)
-        ref = self.domain_config_api.update_config(
+        PROVIDERS.resource_api.get_domain(domain_id)
+        ref = PROVIDERS.domain_config_api.update_config(
             domain_id, config, group, option)
         return wsgi.render_response(body={self.member_name: ref})
 
     def update_domain_config_group(self, context, domain_id, group, config):
-        self.resource_api.get_domain(domain_id)
+        PROVIDERS.resource_api.get_domain(domain_id)
         return self.update_domain_config(
             context, domain_id, config, group, option=None)
 
     def update_domain_config_only(self, context, domain_id, config):
-        self.resource_api.get_domain(domain_id)
+        PROVIDERS.resource_api.get_domain(domain_id)
         return self.update_domain_config(
             context, domain_id, config, group=None, option=None)
 
     @controller.protected()
     def delete_domain_config(
             self, request, domain_id, group=None, option=None):
-        self.resource_api.get_domain(domain_id)
-        self.domain_config_api.delete_config(domain_id, group, option)
+        PROVIDERS.resource_api.get_domain(domain_id)
+        PROVIDERS.domain_config_api.delete_config(domain_id, group, option)
 
     @controller.protected()
     def get_domain_config_default(self, request, group=None, option=None):
-        ref = self.domain_config_api.get_config_default(group, option)
+        ref = PROVIDERS.domain_config_api.get_config_default(group, option)
         return {self.member_name: ref}
 
 
-@dependency.requires('resource_api')
 class ProjectV3(controller.V3Controller):
     collection_name = 'projects'
     member_name = 'project'
 
     def __init__(self):
         super(ProjectV3, self).__init__()
-        self.get_member_from_driver = self.resource_api.get_project
+        self.get_member_from_driver = PROVIDERS.resource_api.get_project
 
     @controller.protected()
     def create_project(self, request, project):
@@ -263,7 +174,7 @@ class ProjectV3(controller.V3Controller):
             ref['parent_id'] = ref.get('domain_id')
 
         try:
-            ref = self.resource_api.create_project(
+            ref = PROVIDERS.resource_api.create_project(
                 ref['id'],
                 ref,
                 initiator=request.audit_initiator)
@@ -279,7 +190,13 @@ class ProjectV3(controller.V3Controller):
         # False (which in query terms means '0')
         if 'is_domain' not in request.params:
             hints.add_filter('is_domain', '0')
-        refs = self.resource_api.list_projects(hints=hints)
+        # If any tags filters are passed in when listing projects, add them
+        # to the hint filters
+        tag_params = ['tags', 'tags-any', 'not-tags', 'not-tags-any']
+        for t in tag_params:
+            if t in request.params:
+                hints.add_filter(t, request.params[t])
+        refs = PROVIDERS.resource_api.list_projects(hints=hints)
         return ProjectV3.wrap_collection(request.context_dict,
                                          refs, hints=hints)
 
@@ -310,25 +227,30 @@ class ProjectV3(controller.V3Controller):
             raise exception.ValidationError(msg)
 
         if parents_as_list:
-            parents = self.resource_api.list_project_parents(
+            parents = PROVIDERS.resource_api.list_project_parents(
                 ref['id'], request.context.user_id)
             ref['parents'] = [ProjectV3.wrap_member(context, p)
                               for p in parents]
         elif parents_as_ids:
-            ref['parents'] = self.resource_api.get_project_parents_as_ids(ref)
+            ref['parents'] = PROVIDERS.resource_api.get_project_parents_as_ids(
+                ref
+            )
 
         if subtree_as_list:
-            subtree = self.resource_api.list_projects_in_subtree(
+            subtree = PROVIDERS.resource_api.list_projects_in_subtree(
                 ref['id'], request.context.user_id)
             ref['subtree'] = [ProjectV3.wrap_member(context, p)
                               for p in subtree]
         elif subtree_as_ids:
-            ref['subtree'] = self.resource_api.get_projects_in_subtree_as_ids(
-                ref['id'])
+            ref['subtree'] = (
+                PROVIDERS.resource_api.get_projects_in_subtree_as_ids(
+                    ref['id']
+                )
+            )
 
     @controller.protected()
     def get_project(self, request, project_id):
-        ref = self.resource_api.get_project(project_id)
+        ref = PROVIDERS.resource_api.get_project(project_id)
         self._expand_project_ref(request, ref)
         return ProjectV3.wrap_member(request.context_dict, ref)
 
@@ -336,7 +258,7 @@ class ProjectV3(controller.V3Controller):
     def update_project(self, request, project_id, project):
         validation.lazy_validate(schema.project_update, project)
         self._require_matching_id(project_id, project)
-        ref = self.resource_api.update_project(
+        ref = PROVIDERS.resource_api.update_project(
             project_id,
             project,
             initiator=request.audit_initiator)
@@ -344,6 +266,71 @@ class ProjectV3(controller.V3Controller):
 
     @controller.protected()
     def delete_project(self, request, project_id):
-        return self.resource_api.delete_project(
+        return PROVIDERS.resource_api.delete_project(
             project_id,
             initiator=request.audit_initiator)
+
+
+class ProjectTagV3(controller.V3Controller):
+    collection_name = 'projects'
+    member_name = 'tags'
+
+    def __init__(self):
+        super(ProjectTagV3, self).__init__()
+        self.get_member_from_driver = PROVIDERS.resource_api.get_project_tag
+
+    @classmethod
+    def wrap_member(cls, context, ref):
+        # NOTE(gagehugo): Overriding this due to how the common controller
+        # expects the ref to have an id, which for tags it does not.
+        new_ref = {'links': {'self': cls.full_url(context)}}
+        new_ref[cls.member_name] = (ref or [])
+        return new_ref
+
+    @classmethod
+    def wrap_header(cls, context, query):
+        # NOTE(gagehugo: The API spec for tags has a specific guideline for
+        # what to return when adding a single tag. This wrapper handles
+        # returning the specified url in the header while the body is empty.
+        context['environment']['QUERY_STRING'] = '/' + query
+        url = cls.full_url(context)
+        headers = [('Location', url.replace('?', ''))]
+        status = (http_client.CREATED,
+                  http_client.responses[http_client.CREATED])
+        return wsgi.render_response(status=status, headers=headers)
+
+    @controller.protected()
+    def create_project_tag(self, request, project_id, value):
+        validation.lazy_validate(schema.project_tag_create, value)
+        # Check if we will exceed the max number of tags on this project
+        tags = PROVIDERS.resource_api.list_project_tags(project_id)
+        tags.append(value)
+        validation.lazy_validate(schema.project_tags_update, tags)
+        PROVIDERS.resource_api.create_project_tag(
+            project_id, value, initiator=request.audit_initiator)
+        query = '/'.join((project_id, 'tags', value))
+        return ProjectTagV3.wrap_header(request.context_dict, query)
+
+    @controller.protected()
+    def get_project_tag(self, request, project_id, value):
+        PROVIDERS.resource_api.get_project_tag(project_id, value)
+
+    @controller.protected()
+    def delete_project_tag(self, request, project_id, value):
+        PROVIDERS.resource_api.delete_project_tag(project_id, value)
+
+    @controller.protected()
+    def list_project_tags(self, request, project_id):
+        ref = PROVIDERS.resource_api.list_project_tags(project_id)
+        return ProjectTagV3.wrap_member(request.context_dict, ref)
+
+    @controller.protected()
+    def update_project_tags(self, request, project_id, tags):
+        validation.lazy_validate(schema.project_tags_update, tags)
+        ref = PROVIDERS.resource_api.update_project_tags(
+            project_id, tags, initiator=request.audit_initiator)
+        return ProjectTagV3.wrap_member(request.context_dict, ref)
+
+    @controller.protected()
+    def delete_project_tags(self, request, project_id):
+        PROVIDERS.resource_api.update_project_tags(project_id, [])
