@@ -26,7 +26,7 @@ class JsonHomeTest(unit.BaseTestCase):
         resource_name = self.getUniqueString()
         relation = json_home.build_v3_resource_relation(resource_name)
         exp_relation = (
-            'http://docs.openstack.org/api/openstack-identity/3/rel/%s' %
+            'https://docs.openstack.org/api/openstack-identity/3/rel/%s' %
             resource_name)
         self.assertThat(relation, matchers.Equals(exp_relation))
 
@@ -37,15 +37,15 @@ class JsonHomeTest(unit.BaseTestCase):
         relation = json_home.build_v3_extension_resource_relation(
             extension_name, extension_version, resource_name)
         exp_relation = (
-            'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/rel/'
-            '%s' % (extension_name, extension_version, resource_name))
+            'https://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/rel'
+            '/%s' % (extension_name, extension_version, resource_name))
         self.assertThat(relation, matchers.Equals(exp_relation))
 
     def test_build_v3_parameter_relation(self):
         parameter_name = self.getUniqueString()
         relation = json_home.build_v3_parameter_relation(parameter_name)
         exp_relation = (
-            'http://docs.openstack.org/api/openstack-identity/3/param/%s' %
+            'https://docs.openstack.org/api/openstack-identity/3/param/%s' %
             parameter_name)
         self.assertThat(relation, matchers.Equals(exp_relation))
 
@@ -56,7 +56,7 @@ class JsonHomeTest(unit.BaseTestCase):
         relation = json_home.build_v3_extension_parameter_relation(
             extension_name, extension_version, parameter_name)
         exp_relation = (
-            'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/'
+            'https://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/'
             'param/%s' % (extension_name, extension_version, parameter_name))
         self.assertThat(relation, matchers.Equals(exp_relation))
 
