@@ -41,7 +41,7 @@ import testtools
 from testtools import testcase
 
 from keystone.common import context
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import request
 from keystone.common import sql
 import keystone.conf
@@ -74,6 +74,7 @@ def _calc_tmpdir():
 TMPDIR = _calc_tmpdir()
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 log.register_options(CONF)
 
 IN_MEM_DB_CONN_STRING = 'sqlite://'
@@ -273,7 +274,8 @@ def new_domain_ref(**kwargs):
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
         'description': uuid.uuid4().hex,
-        'enabled': True
+        'enabled': True,
+        'tags': []
     }
     ref.update(kwargs)
     return ref
@@ -287,6 +289,7 @@ def new_project_ref(domain_id=None, is_domain=False, **kwargs):
         'enabled': True,
         'domain_id': domain_id,
         'is_domain': is_domain,
+        'tags': []
     }
     # NOTE(henry-nash): We don't include parent_id in the initial list above
     # since specifying it is optional depending on where the project sits in
@@ -459,6 +462,29 @@ def new_trust_ref(trustor_user_id, trustee_user_id, project_id=None,
     return ref
 
 
+def new_registered_limit_ref(**kwargs):
+    ref = {
+        'service_id': uuid.uuid4().hex,
+        'resource_name': uuid.uuid4().hex,
+        'default_limit': 10
+    }
+
+    ref.update(kwargs)
+    return ref
+
+
+def new_limit_ref(**kwargs):
+    ref = {
+        'project_id': uuid.uuid4().hex,
+        'service_id': uuid.uuid4().hex,
+        'resource_name': uuid.uuid4().hex,
+        'resource_limit': 10
+    }
+
+    ref.update(kwargs)
+    return ref
+
+
 def create_user(api, domain_id, **kwargs):
     """Create a user via the API. Keep the created password.
 
@@ -675,7 +701,7 @@ class TestCase(BaseTestCase):
 
         # Clear the registry of providers so that providers from previous
         # tests aren't used.
-        self.addCleanup(dependency.reset)
+        self.addCleanup(provider_api.ProviderAPIs._clear_registry_instances)
 
         # Ensure Notification subscriptions and resource types are empty
         self.addCleanup(notifications.clear_subscribers)
@@ -689,6 +715,10 @@ class TestCase(BaseTestCase):
 
     def load_backends(self):
         """Initialize each manager and assigns them to an attribute."""
+        # TODO(morgan): Ensure our tests only ever call load_backends
+        # a single time via this method. for now just clear the registry
+        # if we are reloading.
+        provider_api.ProviderAPIs._clear_registry_instances()
         self.useFixture(ksfixtures.BackendLoader(self))
 
     def load_fixtures(self, fixtures):
@@ -708,20 +738,20 @@ class TestCase(BaseTestCase):
             hasattr(self, 'assignment_api') and
                 hasattr(self, 'resource_api')):
             for domain in fixtures.DOMAINS:
-                rv = self.resource_api.create_domain(domain['id'], domain)
+                rv = PROVIDERS.resource_api.create_domain(domain['id'], domain)
                 attrname = 'domain_%s' % domain['id']
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
 
             for tenant in fixtures.TENANTS:
                 tenant_attr_name = 'tenant_%s' % tenant['name'].lower()
-                rv = self.resource_api.create_project(
+                rv = PROVIDERS.resource_api.create_project(
                     tenant['id'], tenant)
                 setattr(self, tenant_attr_name, rv)
                 fixtures_to_cleanup.append(tenant_attr_name)
 
             for role in fixtures.ROLES:
-                rv = self.role_api.create_role(role['id'], role)
+                rv = PROVIDERS.role_api.create_role(role['id'], role)
                 attrname = 'role_%s' % role['name']
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
@@ -731,15 +761,17 @@ class TestCase(BaseTestCase):
                 tenants = user_copy.pop('tenants')
 
                 # For users, the manager layer will generate the ID
-                user_copy = self.identity_api.create_user(user_copy)
+                user_copy = PROVIDERS.identity_api.create_user(user_copy)
                 # Our tests expect that the password is still in the user
                 # record so that they can reference it, so put it back into
                 # the dict returned.
                 user_copy['password'] = user['password']
 
+                # fixtures.ROLES[2] is the _member_ role.
                 for tenant_id in tenants:
-                    self.assignment_api.add_user_to_project(
-                        tenant_id, user_copy['id'])
+                    PROVIDERS.assignment_api.add_role_to_user_and_project(
+                        user_copy['id'], tenant_id, fixtures.ROLES[2]['id'])
+
                 # Use the ID from the fixture as the attribute name, so
                 # that our tests can easily reference each user dict, while
                 # the ID in the dict will be the real public ID.
@@ -752,7 +784,7 @@ class TestCase(BaseTestCase):
                 user = role_assignment['user']
                 tenant_id = role_assignment['tenant_id']
                 user_id = getattr(self, 'user_%s' % user)['id']
-                self.assignment_api.add_role_to_user_and_project(
+                PROVIDERS.assignment_api.add_role_to_user_and_project(
                     user_id, tenant_id, role_id)
 
             self.addCleanup(self.cleanup_instance(*fixtures_to_cleanup))
