@@ -23,7 +23,6 @@ import six
 from keystone.common import wsgi
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _LE
 from keystone.tests import unit
 
 
@@ -293,17 +292,17 @@ class TestSecurityErrorTranslation(unit.BaseTestCase):
             setattr, exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
 
     class CustomSecurityError(exception.SecurityError):
-        message_format = _LE('We had a failure in the %(place)r')
+        message_format = 'We had a failure in the %(place)r'
 
     class CustomError(exception.Error):
-        message_format = _LE('We had a failure in the %(place)r')
+        message_format = 'We had a failure in the %(place)r'
 
     def test_nested_translation_of_SecurityErrors(self):
         e = self.CustomSecurityError(place='code')
-        _LE('Admiral found this in the log: %s') % e
+        ('Admiral found this in the log: %s') % e
         self.assertNotIn('programmer error', self.warning_log.output)
 
     def test_that_regular_Errors_can_be_deep_copied(self):
         e = self.CustomError(place='code')
-        _LE('Admiral found this in the log: %s') % e
+        ('Admiral found this in the log: %s') % e
         self.assertNotIn('programmer error', self.warning_log.output)
