@@ -80,27 +80,15 @@ class TestBlockCommentsBeginWithASpace(BaseStyleCheck):
         self.assert_has_errors(code, expected_errors=errors)
 
 
-class TestAssertingNoneEquality(BaseStyleCheck):
+class TestTranslationChecks(BaseStyleCheck):
 
     def get_checker(self):
-        return checks.CheckForAssertingNoneEquality
-
-    def test(self):
-        code = self.code_ex.asserting_none_equality['code']
-        errors = self.code_ex.asserting_none_equality['expected_errors']
-        self.assert_has_errors(code, expected_errors=errors)
-
-
-class BaseLoggingCheck(BaseStyleCheck):
-
-    def get_checker(self):
-        return checks.CheckForLoggingIssues
+        return checks.CheckForTranslationIssues
 
     def get_fixture(self):
-        return hacking_fixtures.HackingLogging()
+        return hacking_fixtures.HackingTranslations()
 
     def assert_has_errors(self, code, expected_errors=None):
-
         # pull out the parts of the error that we'll match against
         actual_errors = (e[:3] for e in self.run_check(code))
         # adjust line numbers to make the fixture data more readable.
@@ -109,27 +97,6 @@ class BaseLoggingCheck(BaseStyleCheck):
                          for e in actual_errors]
         self.assertEqual(expected_errors or [], actual_errors)
 
-
-class TestCheckForDebugLoggingIssues(BaseLoggingCheck):
-
-    def test_for_translations(self):
-        fixture = self.code_ex.assert_no_translations_for_debug_logging
-        code = self.code_ex.shared_imports + fixture['code']
-        errors = fixture['expected_errors']
-        self.assert_has_errors(code, expected_errors=errors)
-
-
-class TestLoggingWithWarn(BaseLoggingCheck):
-
-    def test(self):
-        data = self.code_ex.assert_not_using_deprecated_warn
-        code = self.code_ex.shared_imports + data['code']
-        errors = data['expected_errors']
-        self.assert_has_errors(code, expected_errors=errors)
-
-
-class TestCheckForNonDebugLoggingIssues(BaseLoggingCheck):
-
     def test_for_translations(self):
         for example in self.code_ex.examples:
             code = self.code_ex.shared_imports + example['code']
