@@ -21,7 +21,9 @@ from six.moves import http_client
 import webtest
 
 from keystone.common import authorization
+from keystone.common import provider_api
 from keystone.common import tokenless_auth
+from keystone.common import wsgi
 import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
@@ -32,6 +34,7 @@ from keystone.tests.unit import test_backend_sql
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class MiddlewareRequestTestBase(unit.TestCase):
@@ -105,9 +108,9 @@ class TokenAuthMiddlewareTest(MiddlewareRequestTestBase):
     MIDDLEWARE_CLASS = middleware.TokenAuthMiddleware
 
     def test_request(self):
-        headers = {middleware.AUTH_TOKEN_HEADER: 'MAGIC'}
+        headers = {authorization.AUTH_TOKEN_HEADER: 'MAGIC'}
         req = self._do_middleware_request(headers=headers)
-        context = req.environ[middleware.CONTEXT_ENV]
+        context = req.environ[wsgi.CONTEXT_ENV]
         self.assertEqual('MAGIC', context['token_id'])
 
 
@@ -122,7 +125,7 @@ class JsonBodyMiddlewareTest(MiddlewareRequestTestBase):
                                           headers=headers,
                                           method='post')
         self.assertEqual({"arg1": "one", "arg2": ["a"]},
-                         req.environ[middleware.PARAMS_ENV])
+                         req.environ[wsgi.PARAMS_ENV])
 
     def test_malformed_json(self):
         headers = {'Content-Type': 'application/json'}
@@ -147,7 +150,7 @@ class JsonBodyMiddlewareTest(MiddlewareRequestTestBase):
                                           headers=headers,
                                           method='post')
         self.assertEqual({"arg1": "one", "arg2": ["a"]},
-                         req.environ[middleware.PARAMS_ENV])
+                         req.environ[wsgi.PARAMS_ENV])
 
     def test_unrecognized_content_type(self):
         headers = {'Content-Type': 'text/plain'}
@@ -159,7 +162,7 @@ class JsonBodyMiddlewareTest(MiddlewareRequestTestBase):
     def test_unrecognized_content_type_without_body(self):
         headers = {'Content-Type': 'text/plain'}
         req = self._do_middleware_request(headers=headers)
-        self.assertEqual({}, req.environ.get(middleware.PARAMS_ENV, {}))
+        self.assertEqual({}, req.environ.get(wsgi.PARAMS_ENV, {}))
 
 
 class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
@@ -189,44 +192,45 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
         self.domain = unit.new_domain_ref()
         self.domain_id = self.domain['id']
         self.domain_name = self.domain['name']
-        self.resource_api.create_domain(self.domain_id, self.domain)
+        PROVIDERS.resource_api.create_domain(self.domain_id, self.domain)
 
         # 2) Create a project for the user.
         self.project = unit.new_project_ref(domain_id=self.domain_id)
         self.project_id = self.project['id']
         self.project_name = self.project['name']
 
-        self.resource_api.create_project(self.project_id, self.project)
+        PROVIDERS.resource_api.create_project(self.project_id, self.project)
 
         # 3) Create a user in new domain.
         self.user = unit.new_user_ref(domain_id=self.domain_id,
                                       project_id=self.project_id)
 
-        self.user = self.identity_api.create_user(self.user)
+        self.user = PROVIDERS.identity_api.create_user(self.user)
 
         # Add IDP
         self.idp = self._idp_ref(id=self.idp_id)
-        self.federation_api.create_idp(self.idp['id'],
-                                       self.idp)
+        PROVIDERS.federation_api.create_idp(
+            self.idp['id'], self.idp
+        )
 
         # Add a role
         self.role = unit.new_role_ref()
         self.role_id = self.role['id']
         self.role_name = self.role['name']
-        self.role_api.create_role(self.role_id, self.role)
+        PROVIDERS.role_api.create_role(self.role_id, self.role)
 
         # Add a group
         self.group = unit.new_group_ref(domain_id=self.domain_id)
-        self.group = self.identity_api.create_group(self.group)
+        self.group = PROVIDERS.identity_api.create_group(self.group)
 
         # Assign a role to the user on a project
-        self.assignment_api.add_role_to_user_and_project(
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
             user_id=self.user['id'],
             tenant_id=self.project_id,
             role_id=self.role_id)
 
         # Assign a role to the group on a project
-        self.assignment_api.create_grant(
+        PROVIDERS.assignment_api.create_grant(
             role_id=self.role_id,
             group_id=self.group['id'],
             project_id=self.project_id)
@@ -234,14 +238,15 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
     def _load_mapping_rules(self, rules):
         # Add a mapping
         self.mapping = self._mapping_ref(rules=rules)
-        self.federation_api.create_mapping(self.mapping['id'],
-                                           self.mapping)
+        PROVIDERS.federation_api.create_mapping(
+            self.mapping['id'], self.mapping
+        )
         # Add protocols
         self.proto_x509 = self._proto_ref(mapping_id=self.mapping['id'])
         self.proto_x509['id'] = self.protocol_id
-        self.federation_api.create_protocol(self.idp['id'],
-                                            self.proto_x509['id'],
-                                            self.proto_x509)
+        PROVIDERS.federation_api.create_protocol(
+            self.idp['id'], self.proto_x509['id'], self.proto_x509
+        )
 
     def _idp_ref(self, id=None):
         idp = {
@@ -542,7 +547,7 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
         env['SSL_CLIENT_DOMAIN_ID'] = self.domain_id
 
         self.domain['enabled'] = False
-        self.domain = self.resource_api.update_domain(
+        self.domain = PROVIDERS.resource_api.update_domain(
             self.domain['id'], self.domain)
 
         self._load_mapping_rules(
@@ -560,7 +565,9 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
         env['SSL_CLIENT_DOMAIN_ID'] = self.domain_id
 
         self.user['enabled'] = False
-        self.user = self.identity_api.update_user(self.user['id'], self.user)
+        self.user = PROVIDERS.identity_api.update_user(
+            self.user['id'], self.user
+        )
 
         self._load_mapping_rules(
             mapping_fixtures.MAPPING_WITH_USERNAME_AND_DOMAINID)
@@ -720,15 +727,15 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
     def test_admin_token_context(self):
         self.config_fixture.config(admin_token='ADMIN')
         log_fix = self.useFixture(fixtures.FakeLogger())
-        headers = {middleware.AUTH_TOKEN_HEADER: 'ADMIN'}
+        headers = {authorization.AUTH_TOKEN_HEADER: 'ADMIN'}
         req = self._do_middleware_request(headers=headers)
-        self.assertTrue(req.environ[middleware.CONTEXT_ENV]['is_admin'])
+        self.assertTrue(req.environ[wsgi.CONTEXT_ENV]['is_admin'])
         self.assertNotIn('Invalid user token', log_fix.output)
 
     def test_request_non_admin(self):
         self.config_fixture.config(
             admin_token='ADMIN')
         log_fix = self.useFixture(fixtures.FakeLogger())
-        headers = {middleware.AUTH_TOKEN_HEADER: 'NOT-ADMIN'}
+        headers = {authorization.AUTH_TOKEN_HEADER: 'NOT-ADMIN'}
         self._do_middleware_request(headers=headers)
         self.assertIn('Invalid user token', log_fix.output)
