@@ -18,10 +18,13 @@ import uuid
 from six.moves import http_client
 from testtools import matchers
 
+from keystone.common import provider_api
 from keystone.tests import unit
 from keystone.tests.unit.ksfixtures import database
 from keystone.tests.unit import test_v3
 
+PROVIDERS = provider_api.ProviderAPIs
+
 
 class CatalogTestCase(test_v3.RestfulTestCase):
     """Test service & endpoint CRUD."""
@@ -677,58 +680,6 @@ class CatalogTestCase(test_v3.RestfulTestCase):
             '/endpoints/%(endpoint_id)s' % {
                 'endpoint_id': self.endpoint_id})
 
-    def test_create_endpoint_on_v2(self):
-        # clear the v3 endpoint so we only have endpoints created on v2
-        self.delete(
-            '/endpoints/%(endpoint_id)s' % {
-                'endpoint_id': self.endpoint_id})
-
-        # create a v3 endpoint ref, and then tweak it back to a v2-style ref
-        ref = unit.new_endpoint_ref_with_region(service_id=self.service['id'],
-                                                region=uuid.uuid4().hex,
-                                                internalurl=None)
-        del ref['id']
-        del ref['interface']
-        ref['publicurl'] = ref.pop('url')
-        # don't set adminurl to ensure it's absence is handled like internalurl
-
-        # create the endpoint on v2 (using a v3 token)
-        r = self.admin_request(
-            method='POST',
-            path='/v2.0/endpoints',
-            token=self.get_scoped_token(),
-            body={'endpoint': ref})
-        endpoint_v2 = r.result['endpoint']
-
-        # test the endpoint on v3
-        r = self.get('/endpoints')
-        endpoints = self.assertValidEndpointListResponse(r)
-        self.assertEqual(1, len(endpoints))
-        endpoint_v3 = endpoints.pop()
-
-        # these attributes are identical between both APIs
-        self.assertEqual(ref['region'], endpoint_v3['region_id'])
-        self.assertEqual(ref['service_id'], endpoint_v3['service_id'])
-        self.assertEqual(ref['description'], endpoint_v3['description'])
-
-        # a v2 endpoint is not quite the same concept as a v3 endpoint, so they
-        # receive different identifiers
-        self.assertNotEqual(endpoint_v2['id'], endpoint_v3['id'])
-
-        # v2 has a publicurl; v3 has a url + interface type
-        self.assertEqual(ref['publicurl'], endpoint_v3['url'])
-        self.assertEqual('public', endpoint_v3['interface'])
-
-        # tests for bug 1152632 -- these attributes were being returned by v3
-        self.assertNotIn('publicurl', endpoint_v3)
-        self.assertNotIn('adminurl', endpoint_v3)
-        self.assertNotIn('internalurl', endpoint_v3)
-
-        # test for bug 1152635 -- this attribute was being returned by v3
-        self.assertNotIn('legacy_endpoint_id', endpoint_v3)
-
-        self.assertEqual(endpoint_v2['region'], endpoint_v3['region_id'])
-
     def test_deleting_endpoint_with_space_in_url(self):
         # add a space to all urls (intentional "i d" to test bug)
         url_with_space = "http://127.0.0.1:8774 /v1.1/\$(tenant_i d)s"
@@ -742,7 +693,7 @@ class CatalogTestCase(test_v3.RestfulTestCase):
                                     url=url_with_space)
 
         # add the endpoint to the database
-        self.catalog_api.create_endpoint(ref['id'], ref)
+        PROVIDERS.catalog_api.create_endpoint(ref['id'], ref)
 
         # delete the endpoint
         self.delete('/endpoints/%s' % ref['id'])
@@ -797,6 +748,76 @@ class CatalogTestCase(test_v3.RestfulTestCase):
                       expected_status=http_client.BAD_REQUEST)
 
 
+class TestMultiRegion(test_v3.RestfulTestCase):
+
+    def test_catalog_with_multi_region_reports_all_endpoints(self):
+
+        # Create two separate regions
+        first_region = self.post(
+            '/regions',
+            body={'region': unit.new_region_ref()}
+        ).json_body['region']
+
+        second_region = self.post(
+            '/regions',
+            body={'region': unit.new_region_ref()}
+        ).json_body['region']
+
+        # Create two services with the same type but separate name.
+        first_service = self.post(
+            '/services',
+            body={'service': unit.new_service_ref(type='foobar')}
+        ).json_body['service']
+
+        second_service = self.post(
+            '/services',
+            body={'service': unit.new_service_ref(type='foobar')}
+        ).json_body['service']
+
+        # Create an endpoint for each service
+        first_endpoint = self.post(
+            '/endpoints',
+            body={
+                'endpoint': unit.new_endpoint_ref(
+                    first_service['id'],
+                    interface='public',
+                    region_id=first_region['id']
+                )
+            }
+        ).json_body['endpoint']
+
+        second_endpoint = self.post(
+            '/endpoints',
+            body={
+                'endpoint': unit.new_endpoint_ref(
+                    second_service['id'],
+                    interface='public',
+                    region_id=second_region['id']
+                )
+            }
+        ).json_body['endpoint']
+
+        # Assert the endpoints and services from each region are in the
+        # catalog.
+        found_first_endpoint = False
+        found_second_endpoint = False
+        catalog = self.get('/auth/catalog/').json_body['catalog']
+        for service in catalog:
+            if service['id'] == first_service['id']:
+                endpoint = service['endpoints'][0]
+                self.assertEqual(endpoint['id'], first_endpoint['id'])
+                self.assertEqual(endpoint['region_id'], first_region['id'])
+                found_first_endpoint = True
+            elif service['id'] == second_service['id']:
+                endpoint = service['endpoints'][0]
+                self.assertEqual(endpoint['id'], second_endpoint['id'])
+                self.assertEqual(endpoint['region_id'], second_region['id'])
+                found_second_endpoint = True
+
+        self.assertTrue(found_first_endpoint)
+        self.assertTrue(found_second_endpoint)
+
+
 class TestCatalogAPISQL(unit.TestCase):
     """Test for the catalog Manager against the SQL backend."""
 
@@ -807,7 +828,7 @@ class TestCatalogAPISQL(unit.TestCase):
 
         service = unit.new_service_ref()
         self.service_id = service['id']
-        self.catalog_api.create_service(self.service_id, service)
+        PROVIDERS.catalog_api.create_service(self.service_id, service)
 
         self.create_endpoint(service_id=self.service_id)
 
@@ -815,7 +836,7 @@ class TestCatalogAPISQL(unit.TestCase):
         endpoint = unit.new_endpoint_ref(service_id=service_id,
                                          region_id=None, **kwargs)
 
-        self.catalog_api.create_endpoint(endpoint['id'], endpoint)
+        PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
         return endpoint
 
     def config_overrides(self):
@@ -829,15 +850,15 @@ class TestCatalogAPISQL(unit.TestCase):
         # filter the catalog by the project or replace the url with a
         # valid project id.
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         project = unit.new_project_ref(domain_id=domain['id'])
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
 
         # the only endpoint in the catalog is the one created in setUp
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
         self.assertEqual(1, len(catalog[0]['endpoints']))
         # it's also the only endpoint in the backend
-        self.assertEqual(1, len(self.catalog_api.list_endpoints()))
+        self.assertEqual(1, len(PROVIDERS.catalog_api.list_endpoints()))
 
         # create a new, invalid endpoint - malformed type declaration
         self.create_endpoint(self.service_id,
@@ -848,23 +869,23 @@ class TestCatalogAPISQL(unit.TestCase):
                              url='http://keystone/%(you_wont_find_me)s')
 
         # verify that the invalid endpoints don't appear in the catalog
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
         self.assertEqual(1, len(catalog[0]['endpoints']))
         # all three appear in the backend
-        self.assertEqual(3, len(self.catalog_api.list_endpoints()))
+        self.assertEqual(3, len(PROVIDERS.catalog_api.list_endpoints()))
 
         # create another valid endpoint - project_id will be replaced
         self.create_endpoint(self.service_id,
                              url='http://keystone/%(project_id)s')
 
         # there are two valid endpoints, positive check
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
         self.assertThat(catalog[0]['endpoints'], matchers.HasLength(2))
 
         # If the URL has no 'project_id' to substitute, we will skip the
         # endpoint which contains this kind of URL, negative check.
         project_id = None
-        catalog = self.catalog_api.get_v3_catalog(user_id, project_id)
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project_id)
         self.assertThat(catalog[0]['endpoints'], matchers.HasLength(1))
 
     def test_get_catalog_always_returns_service_name(self):
@@ -873,22 +894,22 @@ class TestCatalogAPISQL(unit.TestCase):
         # filter the catalog by the project or replace the url with a
         # valid project id.
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         project = unit.new_project_ref(domain_id=domain['id'])
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
 
         # create a service, with a name
         named_svc = unit.new_service_ref()
-        self.catalog_api.create_service(named_svc['id'], named_svc)
+        PROVIDERS.catalog_api.create_service(named_svc['id'], named_svc)
         self.create_endpoint(service_id=named_svc['id'])
 
         # create a service, with no name
         unnamed_svc = unit.new_service_ref(name=None)
         del unnamed_svc['name']
-        self.catalog_api.create_service(unnamed_svc['id'], unnamed_svc)
+        PROVIDERS.catalog_api.create_service(unnamed_svc['id'], unnamed_svc)
         self.create_endpoint(service_id=unnamed_svc['id'])
 
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
 
         named_endpoint = [ep for ep in catalog
                           if ep['type'] == named_svc['type']][0]
@@ -916,46 +937,46 @@ class TestCatalogAPISQLRegions(unit.TestCase):
     def test_get_catalog_returns_proper_endpoints_with_no_region(self):
         service = unit.new_service_ref()
         service_id = service['id']
-        self.catalog_api.create_service(service_id, service)
+        PROVIDERS.catalog_api.create_service(service_id, service)
 
         endpoint = unit.new_endpoint_ref(service_id=service_id,
                                          region_id=None)
         del endpoint['region_id']
-        self.catalog_api.create_endpoint(endpoint['id'], endpoint)
+        PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
 
         # create a project since the project should exist if we want to
         # filter the catalog by the project or replace the url with a
         # valid project id.
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         project = unit.new_project_ref(domain_id=domain['id'])
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
         user_id = uuid.uuid4().hex
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
         self.assertValidCatalogEndpoint(
             catalog[0]['endpoints'][0], ref=endpoint)
 
     def test_get_catalog_returns_proper_endpoints_with_region(self):
         service = unit.new_service_ref()
         service_id = service['id']
-        self.catalog_api.create_service(service_id, service)
+        PROVIDERS.catalog_api.create_service(service_id, service)
 
         endpoint = unit.new_endpoint_ref(service_id=service_id)
         region = unit.new_region_ref(id=endpoint['region_id'])
-        self.catalog_api.create_region(region)
-        self.catalog_api.create_endpoint(endpoint['id'], endpoint)
+        PROVIDERS.catalog_api.create_region(region)
+        PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
 
-        endpoint = self.catalog_api.get_endpoint(endpoint['id'])
+        endpoint = PROVIDERS.catalog_api.get_endpoint(endpoint['id'])
         user_id = uuid.uuid4().hex
         # create a project since the project should exist if we want to
         # filter the catalog by the project or replace the url with a
         # valid project id.
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
         project = unit.new_project_ref(domain_id=domain['id'])
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
 
-        catalog = self.catalog_api.get_v3_catalog(user_id, project['id'])
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(user_id, project['id'])
         self.assertValidCatalogEndpoint(
             catalog[0]['endpoints'][0], ref=endpoint)
 
