@@ -16,7 +16,7 @@ from six.moves import http_client
 from six.moves import range
 from testtools import matchers
 
-from keystone.common import controller
+from keystone.common import provider_api
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
@@ -27,6 +27,7 @@ from keystone.tests.unit import utils as test_utils
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class ResourceTestCase(test_v3.RestfulTestCase,
@@ -226,31 +227,23 @@ class ResourceTestCase(test_v3.RestfulTestCase,
         """Call ``PATCH /domains/{domain_id}`` (set enabled=False)."""
         # Create a 2nd set of entities in a 2nd domain
         domain2 = unit.new_domain_ref()
-        self.resource_api.create_domain(domain2['id'], domain2)
+        PROVIDERS.resource_api.create_domain(domain2['id'], domain2)
 
         project2 = unit.new_project_ref(domain_id=domain2['id'])
-        self.resource_api.create_project(project2['id'], project2)
+        PROVIDERS.resource_api.create_project(project2['id'], project2)
 
-        user2 = unit.create_user(self.identity_api,
+        user2 = unit.create_user(PROVIDERS.identity_api,
                                  domain_id=domain2['id'],
                                  project_id=project2['id'])
 
-        self.assignment_api.add_user_to_project(project2['id'],
-                                                user2['id'])
+        role_member = unit.new_role_ref()
+        PROVIDERS.role_api.create_role(role_member['id'], role_member)
+
+        PROVIDERS.assignment_api.add_role_to_user_and_project(
+            user2['id'], project2['id'], role_member['id']
+        )
 
         # First check a user in that domain can authenticate..
-        body = {
-            'auth': {
-                'passwordCredentials': {
-                    'userId': user2['id'],
-                    'password': user2['password']
-                },
-                'tenantId': project2['id']
-            }
-        }
-        self.admin_request(
-            path='/v2.0/tokens', method='POST', body=body)
-
         auth_data = self.build_authentication_request(
             user_id=user2['id'],
             password=user2['password'],
@@ -264,21 +257,6 @@ class ResourceTestCase(test_v3.RestfulTestCase,
             body={'domain': {'enabled': False}})
         self.assertValidDomainResponse(r, domain2)
 
-        # Make sure the user can no longer authenticate, via
-        # either API
-        body = {
-            'auth': {
-                'passwordCredentials': {
-                    'userId': user2['id'],
-                    'password': user2['password']
-                },
-                'tenantId': project2['id']
-            }
-        }
-        self.admin_request(
-            path='/v2.0/tokens', method='POST', body=body,
-            expected_status=http_client.UNAUTHORIZED)
-
         # Try looking up in v3 by name and id
         auth_data = self.build_authentication_request(
             user_id=user2['id'],
@@ -324,30 +302,35 @@ class ResourceTestCase(test_v3.RestfulTestCase,
         """
         # Create a group and a credential in the main domain
         group = unit.new_group_ref(domain_id=self.domain_id)
-        group = self.identity_api.create_group(group)
+        group = PROVIDERS.identity_api.create_group(group)
 
         credential = unit.new_credential_ref(user_id=self.user['id'],
                                              project_id=self.project_id)
-        self.credential_api.create_credential(credential['id'], credential)
+        PROVIDERS.credential_api.create_credential(
+            credential['id'], credential
+        )
 
         # Create a 2nd set of entities in a 2nd domain
         domain2 = unit.new_domain_ref()
-        self.resource_api.create_domain(domain2['id'], domain2)
+        PROVIDERS.resource_api.create_domain(domain2['id'], domain2)
 
         project2 = unit.new_project_ref(domain_id=domain2['id'])
-        project2 = self.resource_api.create_project(project2['id'], project2)
+        project2 = PROVIDERS.resource_api.create_project(
+            project2['id'], project2
+        )
 
         user2 = unit.new_user_ref(domain_id=domain2['id'],
                                   project_id=project2['id'])
-        user2 = self.identity_api.create_user(user2)
+        user2 = PROVIDERS.identity_api.create_user(user2)
 
         group2 = unit.new_group_ref(domain_id=domain2['id'])
-        group2 = self.identity_api.create_group(group2)
+        group2 = PROVIDERS.identity_api.create_group(group2)
 
         credential2 = unit.new_credential_ref(user_id=user2['id'],
                                               project_id=project2['id'])
-        self.credential_api.create_credential(credential2['id'],
-                                              credential2)
+        PROVIDERS.credential_api.create_credential(
+            credential2['id'], credential2
+        )
 
         # Now disable the new domain and delete it
         domain2['enabled'] = False
@@ -359,32 +342,32 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         # Check all the domain2 relevant entities are gone
         self.assertRaises(exception.DomainNotFound,
-                          self.resource_api.get_domain,
+                          PROVIDERS.resource_api.get_domain,
                           domain2['id'])
         self.assertRaises(exception.ProjectNotFound,
-                          self.resource_api.get_project,
+                          PROVIDERS.resource_api.get_project,
                           project2['id'])
         self.assertRaises(exception.GroupNotFound,
-                          self.identity_api.get_group,
+                          PROVIDERS.identity_api.get_group,
                           group2['id'])
         self.assertRaises(exception.UserNotFound,
-                          self.identity_api.get_user,
+                          PROVIDERS.identity_api.get_user,
                           user2['id'])
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           credential2['id'])
 
         # ...and that all self.domain entities are still here
-        r = self.resource_api.get_domain(self.domain['id'])
+        r = PROVIDERS.resource_api.get_domain(self.domain['id'])
         self.assertDictEqual(self.domain, r)
-        r = self.resource_api.get_project(self.project['id'])
+        r = PROVIDERS.resource_api.get_project(self.project['id'])
         self.assertDictEqual(self.project, r)
-        r = self.identity_api.get_group(group['id'])
+        r = PROVIDERS.identity_api.get_group(group['id'])
         self.assertDictEqual(group, r)
-        r = self.identity_api.get_user(self.user['id'])
+        r = PROVIDERS.identity_api.get_user(self.user['id'])
         self.user.pop('password')
         self.assertDictEqual(self.user, r)
-        r = self.credential_api.get_credential(credential['id'])
+        r = PROVIDERS.credential_api.get_credential(credential['id'])
         self.assertDictEqual(credential, r)
 
     def test_delete_domain_deletes_is_domain_project(self):
@@ -428,9 +411,9 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         """
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
 
-        user2 = unit.create_user(self.identity_api,
+        user2 = unit.create_user(PROVIDERS.identity_api,
                                  domain_id=domain['id'])
 
         # build a request body
@@ -463,16 +446,17 @@ class ResourceTestCase(test_v3.RestfulTestCase,
     def test_delete_domain_hierarchy(self):
         """Call ``DELETE /domains/{domain_id}``."""
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
 
         root_project = unit.new_project_ref(domain_id=domain['id'])
-        root_project = self.resource_api.create_project(root_project['id'],
-                                                        root_project)
+        root_project = PROVIDERS.resource_api.create_project(
+            root_project['id'], root_project
+        )
 
         leaf_project = unit.new_project_ref(
             domain_id=domain['id'],
             parent_id=root_project['id'])
-        self.resource_api.create_project(leaf_project['id'], leaf_project)
+        PROVIDERS.resource_api.create_project(leaf_project['id'], leaf_project)
 
         # Need to disable it first.
         self.patch('/domains/%(domain_id)s' % {
@@ -484,15 +468,15 @@ class ResourceTestCase(test_v3.RestfulTestCase,
                 'domain_id': domain['id']})
 
         self.assertRaises(exception.DomainNotFound,
-                          self.resource_api.get_domain,
+                          PROVIDERS.resource_api.get_domain,
                           domain['id'])
 
         self.assertRaises(exception.ProjectNotFound,
-                          self.resource_api.get_project,
+                          PROVIDERS.resource_api.get_project,
                           root_project['id'])
 
         self.assertRaises(exception.ProjectNotFound,
-                          self.resource_api.get_project,
+                          PROVIDERS.resource_api.get_project,
                           leaf_project['id'])
 
     def test_forbid_operations_on_federated_domain(self):
@@ -512,26 +496,26 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         for domain in create_domains():
             self.assertRaises(
-                AssertionError, self.resource_api.create_domain,
+                AssertionError, PROVIDERS.resource_api.create_domain,
                 domain['id'], domain)
             self.assertRaises(
-                AssertionError, self.resource_api.update_domain,
+                AssertionError, PROVIDERS.resource_api.update_domain,
                 domain['id'], domain)
             self.assertRaises(
-                exception.DomainNotFound, self.resource_api.delete_domain,
+                exception.DomainNotFound, PROVIDERS.resource_api.delete_domain,
                 domain['id'])
 
             # swap 'name' with 'id' and try again, expecting the request to
             # gracefully fail
             domain['id'], domain['name'] = domain['name'], domain['id']
             self.assertRaises(
-                AssertionError, self.resource_api.create_domain,
+                AssertionError, PROVIDERS.resource_api.create_domain,
                 domain['id'], domain)
             self.assertRaises(
-                AssertionError, self.resource_api.update_domain,
+                AssertionError, PROVIDERS.resource_api.update_domain,
                 domain['id'], domain)
             self.assertRaises(
-                exception.DomainNotFound, self.resource_api.delete_domain,
+                exception.DomainNotFound, PROVIDERS.resource_api.delete_domain,
                 domain['id'])
 
     def test_forbid_operations_on_defined_federated_domain(self):
@@ -545,13 +529,13 @@ class ResourceTestCase(test_v3.RestfulTestCase,
                                    federated_domain_name=non_default_name)
         domain = unit.new_domain_ref(name=non_default_name)
         self.assertRaises(AssertionError,
-                          self.resource_api.create_domain,
+                          PROVIDERS.resource_api.create_domain,
                           domain['id'], domain)
         self.assertRaises(exception.DomainNotFound,
-                          self.resource_api.delete_domain,
+                          PROVIDERS.resource_api.delete_domain,
                           domain['id'])
         self.assertRaises(AssertionError,
-                          self.resource_api.update_domain,
+                          PROVIDERS.resource_api.update_domain,
                           domain['id'], domain)
 
     # Project CRUD tests
@@ -712,6 +696,147 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         return projects
 
+    def _create_project_and_tags(self, num_of_tags=1):
+        """Create a project and a number of tags attached to that project.
+
+        :param num_of_tags: the desired number of tags created with a specified
+                            project.
+
+        :returns: A tuple containing a new project and a list of
+                  random tags
+        """
+        tags = [uuid.uuid4().hex for i in range(num_of_tags)]
+        ref = unit.new_project_ref(
+            domain_id=self.domain_id,
+            tags=tags)
+        resp = self.post('/projects', body={'project': ref})
+        return resp.result['project'], tags
+
+    def test_list_project_response_returns_tags(self):
+        """Call ``GET /projects`` should always return tag attributes."""
+        tagged_project, tags = self._create_project_and_tags()
+        self.get('/projects')
+        ref = unit.new_project_ref(domain_id=self.domain_id)
+        untagged_project = self.post(
+            '/projects', body={'project': ref}
+        ).json_body['project']
+        resp = self.get('/projects')
+        for project in resp.json_body['projects']:
+            if project['id'] == tagged_project['id']:
+                self.assertIsNotNone(project['tags'])
+                self.assertEqual(project['tags'], tags)
+            if project['id'] == untagged_project['id']:
+                self.assertEqual(project['tags'], [])
+
+    def test_list_projects_filtering_by_tags(self):
+        """Call ``GET /projects?tags={tags}``."""
+        project, tags = self._create_project_and_tags(num_of_tags=2)
+        tag_string = ','.join(tags)
+        resp = self.get('/projects?tags=%(values)s' % {
+            'values': tag_string})
+        self.assertValidProjectListResponse(resp)
+        self.assertEqual(project['id'], resp.result['projects'][0]['id'])
+
+    def test_list_projects_filtering_by_tags_any(self):
+        """Call ``GET /projects?tags-any={tags}``."""
+        project, tags = self._create_project_and_tags(num_of_tags=2)
+        resp = self.get('/projects?tags-any=%(values)s' % {
+            'values': tags[0]})
+        self.assertValidProjectListResponse(resp)
+        self.assertEqual(project['id'], resp.result['projects'][0]['id'])
+
+    def test_list_projects_filtering_by_not_tags(self):
+        """Call ``GET /projects?not-tags={tags}``."""
+        project1, tags1 = self._create_project_and_tags(num_of_tags=2)
+        project2, tags2 = self._create_project_and_tags(num_of_tags=2)
+        tag_string = ','.join(tags1)
+        resp = self.get('/projects?not-tags=%(values)s' % {
+            'values': tag_string})
+        self.assertValidProjectListResponse(resp)
+        project_ids = []
+        for project in resp.result['projects']:
+            project_ids.append(project['id'])
+        self.assertNotIn(project1['id'], project_ids)
+        self.assertIn(project2['id'], project_ids)
+
+    def test_list_projects_filtering_by_not_tags_any(self):
+        """Call ``GET /projects?not-tags-any={tags}``."""
+        project1, tags1 = self._create_project_and_tags(num_of_tags=2)
+        project2, tags2 = self._create_project_and_tags(num_of_tags=2)
+        project3, tags3 = self._create_project_and_tags(num_of_tags=2)
+        tag_string = tags1[0] + ',' + tags2[0]
+        resp = self.get('/projects?not-tags-any=%(values)s' % {
+            'values': tag_string})
+        self.assertValidProjectListResponse(resp)
+        project_ids = []
+        for project in resp.result['projects']:
+            project_ids.append(project['id'])
+        self.assertNotIn(project1['id'], project_ids)
+        self.assertNotIn(project2['id'], project_ids)
+        self.assertIn(project3['id'], project_ids)
+
+    def test_list_projects_filtering_multiple_tag_filters(self):
+        """Call ``GET /projects?tags={tags}&tags-any={tags}``."""
+        project1, tags1 = self._create_project_and_tags()
+        project2, tags2 = self._create_project_and_tags(num_of_tags=2)
+        url = '/projects?tags=%(value1)s&tags-any=%(value2)s'
+        resp = self.get(url % {'value1': tags1[0],
+                               'value2': tags2[0]})
+        self.assertValidProjectListResponse(resp)
+        pids = [project1['id'], project2['id']]
+        self.assertEqual(len(resp.result['projects']), 2)
+        for p in resp.result['projects']:
+            self.assertIn(p['id'], pids)
+
+    def test_list_projects_filtering_multiple_any_tag_filters(self):
+        """Call ``GET /projects?tags-any={tags}&not-tags-any={tags}``."""
+        project1, tags1 = self._create_project_and_tags()
+        project2, tags2 = self._create_project_and_tags(num_of_tags=2)
+        url = '/projects?tags-any=%(value1)s&not-tags-any=%(value2)s'
+        resp = self.get(url % {'value1': tags1[0],
+                               'value2': tags2[0]})
+        self.assertValidProjectListResponse(resp)
+        pids = [p['id'] for p in resp.result['projects']]
+        self.assertIn(project1['id'], pids)
+        self.assertNotIn(project2['id'], pids)
+
+    def test_list_projects_filtering_conflict_tag_filters(self):
+        """Call ``GET /projects?tags={tags}&not-tags={tags}``."""
+        project, tags = self._create_project_and_tags(num_of_tags=2)
+        tag_string = ','.join(tags)
+        url = '/projects?tags=%(values)s&not-tags=%(values)s'
+        resp = self.get(url % {'values': tag_string})
+        self.assertValidProjectListResponse(resp)
+        self.assertEqual(len(resp.result['projects']), 0)
+
+    def test_list_projects_filtering_conflict_any_tag_filters(self):
+        """Call ``GET /projects?tags-any={tags}&not-tags-any={tags}``."""
+        project, tags = self._create_project_and_tags(num_of_tags=2)
+        tag_string = ','.join(tags)
+        url = '/projects?tags-any=%(values)s&not-tags-any=%(values)s'
+        resp = self.get(url % {'values': tag_string})
+        self.assertValidProjectListResponse(resp)
+        self.assertEqual(len(resp.result['projects']), 0)
+
+    def test_list_projects_by_tags_and_name(self):
+        """Call ``GET /projects?tags-any={tags}&name={name}``."""
+        project, tags = self._create_project_and_tags(num_of_tags=2)
+        ref = {'project': {'name': 'tags and name'}}
+        resp = self.patch('/projects/%(project_id)s' %
+                          {'project_id': project['id']},
+                          body=ref)
+        url = '/projects?tags-any=%(values)s&name=%(name)s'
+        resp = self.get(url % {'values': tags[0],
+                               'name': 'tags and name'})
+        self.assertValidProjectListResponse(resp)
+        pids = [p['id'] for p in resp.result['projects']]
+        self.assertIn(project['id'], pids)
+
+        resp = self.get(url % {'values': tags[0],
+                               'name': 'foo'})
+        self.assertValidProjectListResponse(resp)
+        self.assertEqual(len(resp.result['projects']), 0)
+
     def test_list_projects_filtering_by_parent_id(self):
         """Call ``GET /projects?parent_id={project_id}``."""
         projects = self._create_projects_hierarchy(hierarchy_size=2)
@@ -936,10 +1061,10 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         # Add some more projects acting as domains
         new_is_domain_project = unit.new_project_ref(is_domain=True)
-        new_is_domain_project = self.resource_api.create_project(
+        new_is_domain_project = PROVIDERS.resource_api.create_project(
             new_is_domain_project['id'], new_is_domain_project)
         new_is_domain_project2 = unit.new_project_ref(is_domain=True)
-        new_is_domain_project2 = self.resource_api.create_project(
+        new_is_domain_project2 = PROVIDERS.resource_api.create_project(
             new_is_domain_project2['id'], new_is_domain_project2)
         number_is_domain_true = initial_number_is_domain_true + 2
 
@@ -953,7 +1078,7 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         # Now add a regular project
         new_regular_project = unit.new_project_ref(domain_id=self.domain_id)
-        new_regular_project = self.resource_api.create_project(
+        new_regular_project = PROVIDERS.resource_api.create_project(
             new_regular_project['id'], new_regular_project)
         number_is_domain_false = initial_number_is_domain_false + 1
 
@@ -977,7 +1102,7 @@ class ResourceTestCase(test_v3.RestfulTestCase,
 
         # Make sure we have at least one project acting as a domain
         new_is_domain_project = unit.new_project_ref(is_domain=True)
-        new_is_domain_project = self.resource_api.create_project(
+        new_is_domain_project = PROVIDERS.resource_api.create_project(
             new_is_domain_project['id'], new_is_domain_project)
 
         r = self.get('/projects', expected_status=200)
@@ -1210,7 +1335,7 @@ class ResourceTestCase(test_v3.RestfulTestCase,
         update the `domain_id` of a project fails.
         """
         project = unit.new_project_ref(domain_id=self.domain['id'])
-        project = self.resource_api.create_project(project['id'], project)
+        project = PROVIDERS.resource_api.create_project(project['id'], project)
         project['domain_id'] = CONF.identity.default_domain_id
         self.patch('/projects/%(project_id)s' % {
             'project_id': project['id']},
@@ -1278,17 +1403,21 @@ class ResourceTestCase(test_v3.RestfulTestCase,
         """
         credential = unit.new_credential_ref(user_id=self.user['id'],
                                              project_id=self.project_id)
-        self.credential_api.create_credential(credential['id'], credential)
+        PROVIDERS.credential_api.create_credential(
+            credential['id'], credential
+        )
 
         # First check the credential for this project is present
-        r = self.credential_api.get_credential(credential['id'])
+        r = PROVIDERS.credential_api.get_credential(credential['id'])
         self.assertDictEqual(credential, r)
         # Create a second credential with a different project
         project2 = unit.new_project_ref(domain_id=self.domain['id'])
-        self.resource_api.create_project(project2['id'], project2)
+        PROVIDERS.resource_api.create_project(project2['id'], project2)
         credential2 = unit.new_credential_ref(user_id=self.user['id'],
                                               project_id=project2['id'])
-        self.credential_api.create_credential(credential2['id'], credential2)
+        PROVIDERS.credential_api.create_credential(
+            credential2['id'], credential2
+        )
 
         # Now delete the project
         self.delete(
@@ -1298,10 +1427,10 @@ class ResourceTestCase(test_v3.RestfulTestCase,
         # Deleting the project should have deleted any credentials
         # that reference this project
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           credential_id=credential['id'])
         # But the credential for project2 is unaffected
-        r = self.credential_api.get_credential(credential2['id'])
+        r = PROVIDERS.credential_api.get_credential(credential2['id'])
         self.assertDictEqual(credential2, r)
 
     def test_delete_not_leaf_project(self):
@@ -1312,138 +1441,224 @@ class ResourceTestCase(test_v3.RestfulTestCase,
                 'project_id': projects[0]['project']['id']},
             expected_status=http_client.FORBIDDEN)
 
+    def test_create_project_with_tags(self):
+        project, tags = self._create_project_and_tags(num_of_tags=10)
+        ref = self.get(
+            '/projects/%(project_id)s' % {
+                'project_id': project['id']},
+            expected_status=http_client.OK)
+        self.assertIn('tags', ref.result['project'])
+        for tag in tags:
+            self.assertIn(tag, ref.result['project']['tags'])
 
-class ResourceV3toV2MethodsTestCase(unit.TestCase):
-    """Test domain V3 to V2 conversion methods."""
+    def test_update_project_with_tags(self):
+        project, tags = self._create_project_and_tags(num_of_tags=9)
+        tag = uuid.uuid4().hex
+        project['tags'].append(tag)
+        ref = self.patch(
+            '/projects/%(project_id)s' % {
+                'project_id': self.project_id},
+            body={'project': {'tags': project['tags']}})
+        self.assertIn(tag, ref.result['project']['tags'])
 
-    def _setup_initial_projects(self):
-        self.project_id = uuid.uuid4().hex
-        self.domain_id = CONF.identity.default_domain_id
-        self.parent_id = uuid.uuid4().hex
-        # Project with only domain_id in ref
-        self.project1 = unit.new_project_ref(id=self.project_id,
-                                             name=self.project_id,
-                                             domain_id=self.domain_id)
-        # Project with both domain_id and parent_id in ref
-        self.project2 = unit.new_project_ref(id=self.project_id,
-                                             name=self.project_id,
-                                             domain_id=self.domain_id,
-                                             parent_id=self.parent_id)
-        # Project with no domain_id and parent_id in ref
-        self.project3 = unit.new_project_ref(id=self.project_id,
-                                             name=self.project_id,
-                                             domain_id=self.domain_id,
-                                             parent_id=self.parent_id)
-        # Expected result with no domain_id and parent_id
-        self.expected_project = {'id': self.project_id,
-                                 'name': self.project_id}
+    def test_create_project_tag(self):
+        tag = uuid.uuid4().hex
+        url = '/projects/%(project_id)s/tags/%(value)s'
+        self.put(url % {'project_id': self.project_id, 'value': tag},
+                 expected_status=http_client.CREATED)
+        self.get(url % {'project_id': self.project_id, 'value': tag},
+                 expected_status=http_client.NO_CONTENT)
 
-    def test_v2controller_filter_domain_id(self):
-        # V2.0 is not domain aware, ensure domain_id is popped off the ref.
-        other_data = uuid.uuid4().hex
-        domain_id = CONF.identity.default_domain_id
-        ref = {'domain_id': domain_id,
-               'other_data': other_data}
+    def test_create_project_tag_is_case_insensitive(self):
+        case_tags = ['case', 'CASE']
+        for tag in case_tags:
+            self.put(
+                '/projects/%(project_id)s/tags/%(value)s' % {
+                    'project_id': self.project_id,
+                    'value': tag},
+                expected_status=http_client.CREATED)
+        resp = self.get('/projects/%(project_id)s' %
+                        {'project_id': self.project_id},
+                        expected_status=http_client.OK)
+        for tag in case_tags:
+            self.assertIn(tag, resp.result['project']['tags'])
 
-        ref_no_domain = {'other_data': other_data}
-        expected_ref = ref_no_domain.copy()
+    def test_get_single_project_tag(self):
+        project, tags = self._create_project_and_tags()
+        self.get(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tags[0]},
+            expected_status=http_client.NO_CONTENT)
+        self.head(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tags[0]},
+            expected_status=http_client.NO_CONTENT)
 
-        updated_ref = controller.V2Controller.filter_domain_id(ref)
-        self.assertIs(ref, updated_ref)
-        self.assertDictEqual(expected_ref, ref)
-        # Make sure we don't error/muck up data if domain_id isn't present
-        updated_ref = controller.V2Controller.filter_domain_id(ref_no_domain)
-        self.assertIs(ref_no_domain, updated_ref)
-        self.assertDictEqual(expected_ref, ref_no_domain)
+    def test_get_project_tag_that_does_not_exist(self):
+        project, _ = self._create_project_and_tags()
+        self.get(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.NOT_FOUND)
 
-    def test_v3controller_filter_domain_id(self):
-        # No data should be filtered out in this case.
-        other_data = uuid.uuid4().hex
-        domain_id = uuid.uuid4().hex
-        ref = {'domain_id': domain_id,
-               'other_data': other_data}
+    def test_delete_project_tag(self):
+        project, tags = self._create_project_and_tags()
+        self.delete(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tags[0]},
+            expected_status=http_client.NO_CONTENT)
+        self.get(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': tags[0]},
+            expected_status=http_client.NOT_FOUND)
 
-        expected_ref = ref.copy()
-        updated_ref = controller.V3Controller.filter_domain_id(ref)
-        self.assertIs(ref, updated_ref)
-        self.assertDictEqual(expected_ref, ref)
+    def test_delete_project_tags(self):
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        self.delete(
+            '/projects/%(project_id)s/tags/' % {
+                'project_id': project['id']},
+            expected_status=http_client.NO_CONTENT)
+        self.get(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': tags[0]},
+            expected_status=http_client.NOT_FOUND)
+        resp = self.get(
+            '/projects/%(project_id)s/tags/' % {
+                'project_id': self.project_id},
+            expected_status=http_client.OK)
+        self.assertEqual(len(resp.result['tags']), 0)
 
-    def test_v2controller_filter_domain(self):
-        other_data = uuid.uuid4().hex
-        domain_id = uuid.uuid4().hex
-        non_default_domain_ref = {'domain': {'id': domain_id},
-                                  'other_data': other_data}
-        default_domain_ref = {'domain': {'id': 'default'},
-                              'other_data': other_data}
-        updated_ref = controller.V2Controller.filter_domain(default_domain_ref)
-        self.assertNotIn('domain', updated_ref)
-        self.assertNotIn(
-            'domain',
-            controller.V2Controller.filter_domain(non_default_domain_ref))
+    def test_create_project_tag_invalid_project_id(self):
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': uuid.uuid4().hex,
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.NOT_FOUND)
 
-    def test_v2controller_filter_project_parent_id(self):
-        # V2.0 is not project hierarchy aware, ensure parent_id is popped off.
-        other_data = uuid.uuid4().hex
-        parent_id = uuid.uuid4().hex
-        ref = {'parent_id': parent_id,
-               'other_data': other_data}
+    def test_create_project_tag_unsafe_name(self):
+        tag = uuid.uuid4().hex + ','
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': tag},
+            expected_status=http_client.BAD_REQUEST)
 
-        ref_no_parent = {'other_data': other_data}
-        expected_ref = ref_no_parent.copy()
+    def test_create_project_tag_already_exists(self):
+        project, tags = self._create_project_and_tags()
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tags[0]},
+            expected_status=http_client.BAD_REQUEST)
 
-        updated_ref = controller.V2Controller.filter_project_parent_id(ref)
-        self.assertIs(ref, updated_ref)
-        self.assertDictEqual(expected_ref, ref)
-        # Make sure we don't error/muck up data if parent_id isn't present
-        updated_ref = controller.V2Controller.filter_project_parent_id(
-            ref_no_parent)
-        self.assertIs(ref_no_parent, updated_ref)
-        self.assertDictEqual(expected_ref, ref_no_parent)
+    def test_create_project_tag_over_tag_limit(self):
+        project, _ = self._create_project_and_tags(num_of_tags=80)
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.BAD_REQUEST)
 
-    def test_v3_to_v2_project_method(self):
-        self._setup_initial_projects()
+    def test_create_project_tag_name_over_character_limit(self):
+        tag = 'a' * 256
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': tag},
+            expected_status=http_client.BAD_REQUEST)
 
-        # TODO(shaleh): these optional fields are not handled well by the
-        # v3_to_v2 code. Manually remove them for now. Eventually update
-        # new_project_ref to not return optional values
-        del self.project1['enabled']
-        del self.project1['description']
-        del self.project2['enabled']
-        del self.project2['description']
-        del self.project3['enabled']
-        del self.project3['description']
+    def test_delete_tag_invalid_project_id(self):
+        self.delete(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': uuid.uuid4().hex,
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.NOT_FOUND)
 
-        updated_project1 = controller.V2Controller.v3_to_v2_project(
-            self.project1)
-        self.assertIs(self.project1, updated_project1)
-        self.assertDictEqual(self.expected_project, self.project1)
-        updated_project2 = controller.V2Controller.v3_to_v2_project(
-            self.project2)
-        self.assertIs(self.project2, updated_project2)
-        self.assertDictEqual(self.expected_project, self.project2)
-        updated_project3 = controller.V2Controller.v3_to_v2_project(
-            self.project3)
-        self.assertIs(self.project3, updated_project3)
-        self.assertDictEqual(self.expected_project, self.project2)
+    def test_delete_project_tag_not_found(self):
+        self.delete(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.NOT_FOUND)
 
-    def test_v3_to_v2_project_method_list(self):
-        self._setup_initial_projects()
-        project_list = [self.project1, self.project2, self.project3]
+    def test_list_project_tags(self):
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        resp = self.get(
+            '/projects/%(project_id)s/tags' % {
+                'project_id': project['id']},
+            expected_status=http_client.OK)
+        for tag in tags:
+            self.assertIn(tag, resp.result['tags'])
 
-        # TODO(shaleh): these optional fields are not handled well by the
-        # v3_to_v2 code. Manually remove them for now. Eventually update
-        # new_project_ref to not return optional values
-        for p in project_list:
-            del p['enabled']
-            del p['description']
-        updated_list = controller.V2Controller.v3_to_v2_project(project_list)
+    def test_check_if_project_tag_exists(self):
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        self.head(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tags[0]},
+            expected_status=http_client.NO_CONTENT)
 
-        self.assertEqual(len(updated_list), len(project_list))
+    def test_list_project_tags_for_project_with_no_tags(self):
+        resp = self.get(
+            '/projects/%(project_id)s/tags' % {
+                'project_id': self.project_id},
+            expected_status=http_client.OK)
+        self.assertEqual([], resp.result['tags'])
 
-        for i, ref in enumerate(updated_list):
-            # Order should not change.
-            self.assertIs(ref, project_list[i])
+    def test_check_project_with_no_tags(self):
+        self.head(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': self.project_id,
+                'value': uuid.uuid4().hex},
+            expected_status=http_client.NOT_FOUND)
 
-        self.assertDictEqual(self.expected_project, self.project1)
-        self.assertDictEqual(self.expected_project, self.project2)
-        self.assertDictEqual(self.expected_project, self.project3)
+    def test_update_project_tags(self):
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        resp = self.put(
+            '/projects/%(project_id)s/tags' % {
+                'project_id': project['id']},
+            body={'tags': tags},
+            expected_status=http_client.OK)
+        self.assertIn(tags[1], resp.result['tags'])
+
+    def test_update_project_tags_removes_previous_tags(self):
+        tag = uuid.uuid4().hex
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        self.put(
+            '/projects/%(project_id)s/tags/%(value)s' % {
+                'project_id': project['id'],
+                'value': tag},
+            expected_status=http_client.CREATED)
+        resp = self.put(
+            '/projects/%(project_id)s/tags' % {
+                'project_id': project['id']},
+            body={'tags': tags},
+            expected_status=http_client.OK)
+        self.assertNotIn(tag, resp.result['tags'])
+        self.assertIn(tags[1], resp.result['tags'])
+
+    def test_update_project_tags_unsafe_names(self):
+        project, tags = self._create_project_and_tags(num_of_tags=5)
+        invalid_chars = [',', '/']
+        for char in invalid_chars:
+            tags[0] = uuid.uuid4().hex + char
+            self.put(
+                '/projects/%(project_id)s/tags' % {
+                    'project_id': project['id']},
+                body={'tags': tags},
+                expected_status=http_client.BAD_REQUEST)
+
+    def test_update_project_tags_with_too_many_tags(self):
+        project, _ = self._create_project_and_tags()
+        tags = [uuid.uuid4().hex for i in range(81)]
+        tags.append(uuid.uuid4().hex)
+        self.put(
+            '/projects/%(project_id)s/tags' % {'project_id': project['id']},
+            body={'tags': tags},
+            expected_status=http_client.BAD_REQUEST)
