@@ -15,12 +15,14 @@ import uuid
 
 from six.moves import http_client
 
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone.tests import unit
 from keystone.tests.unit import test_v3
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class TestTrustOperations(test_v3.RestfulTestCase):
@@ -35,7 +37,7 @@ class TestTrustOperations(test_v3.RestfulTestCase):
     def setUp(self):
         super(TestTrustOperations, self).setUp()
         # create a trustee to delegate stuff to
-        self.trustee_user = unit.create_user(self.identity_api,
+        self.trustee_user = unit.create_user(PROVIDERS.identity_api,
                                              domain_id=self.domain_id)
         self.trustee_user_id = self.trustee_user['id']
 
@@ -45,6 +47,35 @@ class TestTrustOperations(test_v3.RestfulTestCase):
         self.post('/OS-TRUST/trusts', body={'trust': {}},
                   expected_status=http_client.FORBIDDEN)
 
+    def test_create_trust_with_invalid_expiration_fails(self):
+        # create a new trust
+        ref = unit.new_trust_ref(
+            trustor_user_id=self.user_id,
+            trustee_user_id=self.trustee_user_id,
+            project_id=self.project_id,
+            role_ids=[self.role_id])
+
+        ref['expires_at'] = 'bad'
+        self.post(
+            '/OS-TRUST/trusts',
+            body={'trust': ref},
+            expected_status=http_client.BAD_REQUEST
+        )
+
+        ref['expires_at'] = ''
+        self.post(
+            '/OS-TRUST/trusts',
+            body={'trust': ref},
+            expected_status=http_client.BAD_REQUEST
+        )
+
+        ref['expires_at'] = 'Z'
+        self.post(
+            '/OS-TRUST/trusts',
+            body={'trust': ref},
+            expected_status=http_client.BAD_REQUEST
+        )
+
     def test_trust_crud(self):
         # create a new trust
         ref = unit.new_trust_ref(
@@ -151,6 +182,22 @@ class TestTrustOperations(test_v3.RestfulTestCase):
             expected_status=http_client.FORBIDDEN
         )
 
+    def test_create_trust_with_expiration_in_the_past_fails(self):
+        ref = unit.new_trust_ref(
+            trustor_user_id=self.user_id,
+            trustee_user_id=self.trustee_user_id,
+            project_id=self.project_id,
+            impersonation=False,
+            expires='2010-06-04T08:44:31.999999Z',
+            role_ids=[self.role_id]
+        )
+
+        self.post(
+            '/OS-TRUST/trusts',
+            body={'trust': ref},
+            expected_status=http_client.BAD_REQUEST
+        )
+
     def test_delete_trust(self):
         # create a trust
         ref = unit.new_trust_ref(
@@ -251,6 +298,16 @@ class TestTrustOperations(test_v3.RestfulTestCase):
         self.post('/OS-TRUST/trusts', body={'trust': ref},
                   expected_status=http_client.NOT_FOUND)
 
+    def test_create_trust_with_extra_attributes_fails(self):
+        ref = unit.new_trust_ref(trustor_user_id=self.user_id,
+                                 trustee_user_id=self.trustee_user_id,
+                                 project_id=self.project_id,
+                                 role_ids=[self.role_id])
+        ref['roles'].append({'fake_key': 'fake_value'})
+
+        self.post('/OS-TRUST/trusts', body={'trust': ref},
+                  expected_status=http_client.BAD_REQUEST)
+
     def test_create_trust_with_non_existant_role_name_returns_not_found(self):
         ref = unit.new_trust_ref(
             trustor_user_id=self.user_id,
@@ -275,121 +332,6 @@ class TestTrustOperations(test_v3.RestfulTestCase):
         self.post('/OS-TRUST/trusts', body={'trust': ref},
                   expected_status=http_client.BAD_REQUEST)
 
-    def test_validate_trust_scoped_token_against_v2(self):
-        # get a project-scoped token
-        auth_data = self.build_authentication_request(
-            user_id=self.default_domain_user['id'],
-            password=self.default_domain_user['password'],
-            project_id=self.default_domain_project_id)
-        token = self.get_requested_token(auth_data)
-
-        user = unit.new_user_ref(CONF.identity.default_domain_id)
-        trustee = self.identity_api.create_user(user)
-
-        # create a new trust
-        ref = unit.new_trust_ref(
-            trustor_user_id=self.default_domain_user['id'],
-            trustee_user_id=trustee['id'],
-            project_id=self.default_domain_project_id,
-            impersonation=False,
-            expires=dict(minutes=1),
-            role_ids=[self.role_id])
-        r = self.post('/OS-TRUST/trusts', body={'trust': ref}, token=token)
-        trust = self.assertValidTrustResponse(r)
-
-        # get a v3 trust-scoped token as the trustee
-        auth_data = self.build_authentication_request(
-            user_id=trustee['id'],
-            password=user['password'],
-            trust_id=trust['id'])
-        r = self.v3_create_token(auth_data)
-        self.assertValidProjectScopedTokenResponse(
-            r, trustee)
-        token = r.headers.get('X-Subject-Token')
-
-        # now validate the v3 token with v2 API
-        path = '/v2.0/tokens/%s' % (token)
-        self.admin_request(
-            path=path,
-            token=self.get_admin_token(),
-            method='GET'
-        )
-
-    def test_v3_v2_intermix_trustee_not_in_default_domain_failed(self):
-        # get a project-scoped token
-        auth_data = self.build_authentication_request(
-            user_id=self.default_domain_user['id'],
-            password=self.default_domain_user['password'],
-            project_id=self.default_domain_project_id)
-        token = self.get_requested_token(auth_data)
-
-        # create a new trust
-        ref = unit.new_trust_ref(
-            trustor_user_id=self.default_domain_user_id,
-            trustee_user_id=self.trustee_user_id,
-            project_id=self.default_domain_project_id,
-            impersonation=False,
-            expires=dict(minutes=1),
-            role_ids=[self.role_id])
-        r = self.post('/OS-TRUST/trusts', body={'trust': ref}, token=token)
-        trust = self.assertValidTrustResponse(r)
-
-        # get a trust-scoped token as the trustee
-        auth_data = self.build_authentication_request(
-            user_id=self.trustee_user['id'],
-            password=self.trustee_user['password'],
-            trust_id=trust['id'])
-        r = self.v3_create_token(auth_data)
-        self.assertValidProjectScopedTokenResponse(
-            r, self.trustee_user)
-        token = r.headers.get('X-Subject-Token')
-
-        # now validate the v3 token with v2 API
-        path = '/v2.0/tokens/%s' % (token)
-        self.admin_request(
-            path=path, token=self.get_admin_token(),
-            method='GET', expected_status=http_client.UNAUTHORIZED)
-
-    def test_v3_v2_intermix_project_not_in_default_domain_failed(self):
-        # create a trustee in default domain to delegate stuff to
-        trustee_user = unit.create_user(self.identity_api,
-                                        domain_id=test_v3.DEFAULT_DOMAIN_ID)
-        trustee_user_id = trustee_user['id']
-
-        # create a new trust
-        ref = unit.new_trust_ref(
-            trustor_user_id=self.default_domain_user_id,
-            trustee_user_id=trustee_user_id,
-            project_id=self.project_id,
-            impersonation=False,
-            expires=dict(minutes=1),
-            role_ids=[self.role_id])
-
-        # get a project-scoped token as the default_domain_user
-        auth_data = self.build_authentication_request(
-            user_id=self.default_domain_user['id'],
-            password=self.default_domain_user['password'],
-            project_id=self.default_domain_project_id)
-        token = self.get_requested_token(auth_data)
-
-        r = self.post('/OS-TRUST/trusts', body={'trust': ref}, token=token)
-        trust = self.assertValidTrustResponse(r)
-
-        # get a trust-scoped token as the trustee
-        auth_data = self.build_authentication_request(
-            user_id=trustee_user['id'],
-            password=trustee_user['password'],
-            trust_id=trust['id'])
-        r = self.v3_create_token(auth_data)
-        self.assertValidProjectScopedTokenResponse(r, trustee_user)
-        token = r.headers.get('X-Subject-Token')
-
-        # ensure the token is invalid against v2
-        path = '/v2.0/tokens/%s' % (token)
-        self.admin_request(
-            path=path, token=self.get_admin_token(),
-            method='GET', expected_status=http_client.UNAUTHORIZED)
-
     def test_exercise_trust_scoped_token_without_impersonation(self):
         # create a new trust
         ref = unit.new_trust_ref(
@@ -475,7 +417,7 @@ class TestTrustOperations(test_v3.RestfulTestCase):
 
         # create third-party user, which will be trustee in trust created from
         # redelegated trust
-        third_party_trustee = unit.create_user(self.identity_api,
+        third_party_trustee = unit.create_user(PROVIDERS.identity_api,
                                                domain_id=self.domain_id)
         third_party_trustee_id = third_party_trustee['id']
 
@@ -519,7 +461,7 @@ class TestTrustOperations(test_v3.RestfulTestCase):
             expected_status=http_client.NOT_FOUND)
 
         # create another user as the new trustee
-        trustee_user = unit.create_user(self.identity_api,
+        trustee_user = unit.create_user(PROVIDERS.identity_api,
                                         domain_id=self.domain_id)
         trustee_user_id = trustee_user['id']
         # create the trust again
@@ -537,7 +479,7 @@ class TestTrustOperations(test_v3.RestfulTestCase):
         # call the backend method directly to bypass authentication since the
         # user has been deleted.
         self.assertRaises(exception.TrustNotFound,
-                          self.trust_api.get_trust,
+                          PROVIDERS.trust_api.get_trust,
                           trust['id'])
 
     def test_trust_deleted_when_project_deleted(self):
@@ -564,5 +506,75 @@ class TestTrustOperations(test_v3.RestfulTestCase):
         # call the backend method directly to bypass authentication since the
         # user no longer has the assignment on the project.
         self.assertRaises(exception.TrustNotFound,
-                          self.trust_api.get_trust,
+                          PROVIDERS.trust_api.get_trust,
                           trust['id'])
+
+
+class TrustsWithApplicationCredentials(test_v3.RestfulTestCase):
+
+    def setUp(self):
+        super(TrustsWithApplicationCredentials, self).setUp()
+        self.trustee_user = unit.create_user(PROVIDERS.identity_api,
+                                             domain_id=self.domain_id)
+        self.trustee_user_id = self.trustee_user['id']
+
+    def config_overrides(self):
+        super(TrustsWithApplicationCredentials, self).config_overrides()
+        self.config_fixture.config(group='auth',
+                                   methods='password,application_credential')
+
+    def test_create_trust_with_application_credential(self):
+        app_cred = {
+            'id': uuid.uuid4().hex,
+            'user_id': self.user_id,
+            'project_id': self.project_id,
+            'name': uuid.uuid4().hex,
+            'roles': [{'id': self.role_id}],
+            'secret': uuid.uuid4().hex
+        }
+        app_cred_api = PROVIDERS.application_credential_api
+        app_cred_api.create_application_credential(app_cred)
+        auth_data = self.build_authentication_request(
+            app_cred_id=app_cred['id'], secret=app_cred['secret'])
+        token_data = self.v3_create_token(auth_data,
+                                          expected_status=http_client.CREATED)
+        trust_body = unit.new_trust_ref(trustor_user_id=self.user_id,
+                                        trustee_user_id=self.trustee_user_id,
+                                        project_id=self.project_id,
+                                        role_ids=[self.role_id])
+        self.post(
+            path='/OS-TRUST/trusts',
+            body={'trust': trust_body},
+            token=token_data.headers['x-subject-token'],
+            expected_status=http_client.FORBIDDEN)
+
+    def test_delete_trust_with_application_credential(self):
+        ref = unit.new_trust_ref(
+            trustor_user_id=self.user_id,
+            trustee_user_id=self.trustee_user_id,
+            project_id=self.project_id,
+            impersonation=False,
+            expires=dict(minutes=1),
+            role_ids=[self.role_id])
+        r = self.post('/OS-TRUST/trusts', body={'trust': ref})
+        trust = self.assertValidTrustResponse(r, ref)
+
+        app_cred = {
+            'id': uuid.uuid4().hex,
+            'user_id': self.user_id,
+            'project_id': self.project_id,
+            'name': uuid.uuid4().hex,
+            'roles': [{'id': self.role_id}],
+            'secret': uuid.uuid4().hex
+        }
+        app_cred_api = PROVIDERS.application_credential_api
+        app_cred_api.create_application_credential(app_cred)
+        auth_data = self.build_authentication_request(
+            app_cred_id=app_cred['id'], secret=app_cred['secret'])
+        token_data = self.v3_create_token(auth_data,
+                                          expected_status=http_client.CREATED)
+        # delete the trust
+        self.delete(path='/OS-TRUST/trusts/%(trust_id)s' % {
+            'trust_id': trust['id']},
+            token=token_data.headers['x-subject-token'],
+            expected_status=http_client.FORBIDDEN)
