@@ -21,7 +21,8 @@ from oslo_utils import timeutils
 import six
 
 from keystone import auth
-from keystone.common import fernet_utils
+from keystone.common import provider_api
+from keystone.common import token_utils
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
@@ -31,10 +32,11 @@ from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 from keystone.token.providers import common
 from keystone.token.providers import fernet
-from keystone.token.providers.fernet import token_formatters
+from keystone.token import token_formatters
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class TestFernetTokenProvider(unit.TestCase):
@@ -94,17 +96,18 @@ class TestValidate(unit.TestCase):
         # with a simple token.
 
         domain_ref = unit.new_domain_ref()
-        domain_ref = self.resource_api.create_domain(domain_ref['id'],
-                                                     domain_ref)
+        domain_ref = PROVIDERS.resource_api.create_domain(
+            domain_ref['id'], domain_ref
+        )
 
         user_ref = unit.new_user_ref(domain_ref['id'])
-        user_ref = self.identity_api.create_user(user_ref)
+        user_ref = PROVIDERS.identity_api.create_user(user_ref)
 
         method_names = ['password']
-        token_id, token_data_ = self.token_provider_api.issue_token(
+        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names)
 
-        token_data = self.token_provider_api.validate_token(token_id)
+        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
         token = token_data['token']
         self.assertIsInstance(token['audit_ids'], list)
         self.assertIsInstance(token['expires_at'], str)
@@ -126,29 +129,36 @@ class TestValidate(unit.TestCase):
         # when the token has federated info.
 
         domain_ref = unit.new_domain_ref()
-        domain_ref = self.resource_api.create_domain(domain_ref['id'],
-                                                     domain_ref)
+        domain_ref = PROVIDERS.resource_api.create_domain(
+            domain_ref['id'], domain_ref
+        )
 
         user_ref = unit.new_user_ref(domain_ref['id'])
-        user_ref = self.identity_api.create_user(user_ref)
+        user_ref = PROVIDERS.identity_api.create_user(user_ref)
 
         method_names = ['mapped']
 
         group_ids = [uuid.uuid4().hex, ]
-        identity_provider = uuid.uuid4().hex
+        idp_id = uuid.uuid4().hex
+        idp_ref = {
+            'id': idp_id,
+            'description': uuid.uuid4().hex,
+            'enabled': True
+        }
+        self.federation_api.create_idp(idp_id, idp_ref)
         protocol = uuid.uuid4().hex
         auth_context_params = {
             'user_id': user_ref['id'],
             'user_name': user_ref['name'],
             'group_ids': group_ids,
-            federation_constants.IDENTITY_PROVIDER: identity_provider,
+            federation_constants.IDENTITY_PROVIDER: idp_id,
             federation_constants.PROTOCOL: protocol,
         }
         auth_context = auth.core.AuthContext(**auth_context_params)
-        token_id, token_data_ = self.token_provider_api.issue_token(
+        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names, auth_context=auth_context)
 
-        token_data = self.token_provider_api.validate_token(token_id)
+        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
         token = token_data['token']
         exp_user_info = {
             'id': user_ref['id'],
@@ -157,7 +167,7 @@ class TestValidate(unit.TestCase):
                        'name': CONF.federation.federated_domain_name, },
             federation_constants.FEDERATION: {
                 'groups': [{'id': group_id} for group_id in group_ids],
-                'identity_provider': {'id': identity_provider, },
+                'identity_provider': {'id': idp_id, },
                 'protocol': {'id': protocol, },
             },
         }
@@ -168,27 +178,29 @@ class TestValidate(unit.TestCase):
         # when the token has trust info.
 
         domain_ref = unit.new_domain_ref()
-        domain_ref = self.resource_api.create_domain(domain_ref['id'],
-                                                     domain_ref)
+        domain_ref = PROVIDERS.resource_api.create_domain(
+            domain_ref['id'], domain_ref
+        )
 
         user_ref = unit.new_user_ref(domain_ref['id'])
-        user_ref = self.identity_api.create_user(user_ref)
+        user_ref = PROVIDERS.identity_api.create_user(user_ref)
 
         trustor_user_ref = unit.new_user_ref(domain_ref['id'])
-        trustor_user_ref = self.identity_api.create_user(trustor_user_ref)
+        trustor_user_ref = PROVIDERS.identity_api.create_user(trustor_user_ref)
 
         project_ref = unit.new_project_ref(domain_id=domain_ref['id'])
-        project_ref = self.resource_api.create_project(project_ref['id'],
-                                                       project_ref)
+        project_ref = PROVIDERS.resource_api.create_project(
+            project_ref['id'], project_ref
+        )
 
         role_ref = unit.new_role_ref()
-        role_ref = self.role_api.create_role(role_ref['id'], role_ref)
+        role_ref = PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
 
-        self.assignment_api.create_grant(
+        PROVIDERS.assignment_api.create_grant(
             role_ref['id'], user_id=user_ref['id'],
             project_id=project_ref['id'])
 
-        self.assignment_api.create_grant(
+        PROVIDERS.assignment_api.create_grant(
             role_ref['id'], user_id=trustor_user_ref['id'],
             project_id=project_ref['id'])
 
@@ -197,16 +209,17 @@ class TestValidate(unit.TestCase):
         trust_ref = unit.new_trust_ref(
             trustor_user_id, trustee_user_id, project_id=project_ref['id'],
             role_ids=[role_ref['id'], ])
-        trust_ref = self.trust_api.create_trust(trust_ref['id'], trust_ref,
-                                                trust_ref['roles'])
+        trust_ref = PROVIDERS.trust_api.create_trust(
+            trust_ref['id'], trust_ref, trust_ref['roles']
+        )
 
         method_names = ['password']
 
-        token_id, token_data_ = self.token_provider_api.issue_token(
+        token_id, token_data_ = PROVIDERS.token_provider_api.issue_token(
             user_ref['id'], method_names, project_id=project_ref['id'],
             trust=trust_ref)
 
-        token_data = self.token_provider_api.validate_token(token_id)
+        token_data = PROVIDERS.token_provider_api.validate_token(token_id)
         token = token_data['token']
         exp_trust_info = {
             'id': trust_ref['id'],
@@ -221,8 +234,11 @@ class TestValidate(unit.TestCase):
 
         # A uuid string isn't a valid Fernet token.
         token_id = uuid.uuid4().hex
-        self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api.validate_token, token_id)
+        self.assertRaises(
+            exception.TokenNotFound,
+            PROVIDERS.token_provider_api.validate_token,
+            token_id
+        )
 
 
 class TestTokenFormatter(unit.TestCase):
@@ -322,32 +338,35 @@ class TestPayloads(unit.TestCase):
         self.assertEqual(expected_time_str, actual_time_str)
 
     def _test_payload(self, payload_class, exp_user_id=None, exp_methods=None,
-                      exp_project_id=None, exp_domain_id=None,
-                      exp_trust_id=None, exp_federated_info=None,
-                      exp_access_token_id=None):
+                      exp_system=None, exp_project_id=None,
+                      exp_domain_id=None, exp_trust_id=None,
+                      exp_federated_info=None, exp_access_token_id=None,
+                      exp_app_cred_id=None):
         exp_user_id = exp_user_id or uuid.uuid4().hex
         exp_methods = exp_methods or ['password']
         exp_expires_at = utils.isotime(timeutils.utcnow(), subsecond=True)
         exp_audit_ids = [common.random_urlsafe_str()]
 
         payload = payload_class.assemble(
-            exp_user_id, exp_methods, exp_project_id, exp_domain_id,
-            exp_expires_at, exp_audit_ids, exp_trust_id, exp_federated_info,
-            exp_access_token_id)
+            exp_user_id, exp_methods, exp_system, exp_project_id,
+            exp_domain_id, exp_expires_at, exp_audit_ids, exp_trust_id,
+            exp_federated_info, exp_access_token_id, exp_app_cred_id)
 
-        (user_id, methods, project_id,
+        (user_id, methods, system, project_id,
          domain_id, expires_at, audit_ids,
          trust_id, federated_info,
-         access_token_id) = payload_class.disassemble(payload)
+         access_token_id, app_cred_id) = payload_class.disassemble(payload)
 
         self.assertEqual(exp_user_id, user_id)
         self.assertEqual(exp_methods, methods)
         self.assertTimestampsEqual(exp_expires_at, expires_at)
         self.assertEqual(exp_audit_ids, audit_ids)
+        self.assertEqual(exp_system, system)
         self.assertEqual(exp_project_id, project_id)
         self.assertEqual(exp_domain_id, domain_id)
         self.assertEqual(exp_trust_id, trust_id)
         self.assertEqual(exp_access_token_id, access_token_id)
+        self.assertEqual(exp_app_cred_id, app_cred_id)
 
         if exp_federated_info:
             self.assertDictEqual(exp_federated_info, federated_info)
@@ -357,6 +376,10 @@ class TestPayloads(unit.TestCase):
     def test_unscoped_payload(self):
         self._test_payload(token_formatters.UnscopedPayload)
 
+    def test_system_scoped_payload(self):
+        self._test_payload(token_formatters.SystemScopedPayload,
+                           exp_system='all')
+
     def test_project_scoped_payload(self):
         self._test_payload(token_formatters.ProjectScopedPayload,
                            exp_project_id=uuid.uuid4().hex)
@@ -458,6 +481,18 @@ class TestPayloads(unit.TestCase):
                            exp_project_id=uuid.uuid4().hex,
                            exp_access_token_id=uuid.uuid4().hex)
 
+    def test_app_cred_scoped_payload_with_non_uuid_ids(self):
+        self._test_payload(token_formatters.ApplicationCredentialScopedPayload,
+                           exp_user_id='someNonUuidUserId',
+                           exp_project_id='someNonUuidProjectId',
+                           exp_app_cred_id='someNonUuidAppCredId')
+
+    def test_app_cred_scoped_payload_with_16_char_non_uuid_ids(self):
+        self._test_payload(token_formatters.ApplicationCredentialScopedPayload,
+                           exp_user_id='0123456789abcdef',
+                           exp_project_id='0123456789abcdef',
+                           exp_app_cred_id='0123456789abcdef')
+
 
 class TestFernetKeyRotation(unit.TestCase):
     def setUp(self):
@@ -491,7 +526,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
         """
         # Load the keys into a list, keys is list of six.text_type.
-        key_utils = fernet_utils.FernetUtils(
+        key_utils = token_utils.TokenUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -559,7 +594,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate the keys just enough times to fully populate the key
             # repository.
-            key_utils = fernet_utils.FernetUtils(
+            key_utils = token_utils.TokenUtils(
                 CONF.fernet_tokens.key_repository,
                 CONF.fernet_tokens.max_active_keys,
                 'fernet_tokens'
@@ -577,7 +612,7 @@ class TestFernetKeyRotation(unit.TestCase):
 
             # Rotate an additional number of times to ensure that we maintain
             # the desired number of active keys.
-            key_utils = fernet_utils.FernetUtils(
+            key_utils = token_utils.TokenUtils(
                 CONF.fernet_tokens.key_repository,
                 CONF.fernet_tokens.max_active_keys,
                 'fernet_tokens'
@@ -604,7 +639,7 @@ class TestFernetKeyRotation(unit.TestCase):
         # Make sure that the init key repository contains 2 keys
         self.assertRepositoryState(expected_size=2)
 
-        key_utils = fernet_utils.FernetUtils(
+        key_utils = token_utils.TokenUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -615,13 +650,13 @@ class TestFernetKeyRotation(unit.TestCase):
         file_handle = mock_open()
         file_handle.flush.side_effect = IOError('disk full')
 
-        with mock.patch('keystone.common.fernet_utils.open', mock_open):
+        with mock.patch('keystone.common.token_utils.open', mock_open):
             self.assertRaises(IOError, key_utils.rotate_keys)
 
         # Assert that the key repository is unchanged
         self.assertEqual(self.key_repository_size, 2)
 
-        with mock.patch('keystone.common.fernet_utils.open', mock_open):
+        with mock.patch('keystone.common.token_utils.open', mock_open):
             self.assertRaises(IOError, key_utils.rotate_keys)
 
         # Assert that the key repository is still unchanged, even after
@@ -646,7 +681,7 @@ class TestFernetKeyRotation(unit.TestCase):
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '99.bak')
         with open(evil_file, 'w'):
             pass
-        key_utils = fernet_utils.FernetUtils(
+        key_utils = token_utils.TokenUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -678,7 +713,7 @@ class TestLoadKeys(unit.TestCase):
         evil_file = os.path.join(CONF.fernet_tokens.key_repository, '~1')
         with open(evil_file, 'w'):
             pass
-        key_utils = fernet_utils.FernetUtils(
+        key_utils = token_utils.TokenUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
