@@ -13,19 +13,17 @@
 import os
 
 
-from keystone.common import dependency
 from keystone.common import utils as ks_utils
 import keystone.conf
 from keystone.federation import constants as federation_constants
 from keystone.i18n import _
 from keystone.token.providers import common
-from keystone.token.providers.fernet import token_formatters as tf
+from keystone.token import token_formatters as tf
 
 
 CONF = keystone.conf.CONF
 
 
-@dependency.requires('trust_api', 'oauth_api', 'identity_api')
 class Provider(common.BaseProvider):
     def __init__(self, *args, **kwargs):
         super(Provider, self).__init__(*args, **kwargs)
@@ -153,37 +151,6 @@ class Provider(common.BaseProvider):
         self.v3_token_data_helper.populate_roles_for_federated_user(
             token_dict, group_ids, project_id, domain_id, user_id)
 
-    def _extract_v2_token_data(self, token_data):
-        user_id = token_data['access']['user']['id']
-        expires_at = token_data['access']['token']['expires']
-        audit_ids = token_data['access']['token'].get('audit_ids')
-        methods = ['password']
-        if len(audit_ids) > 1:
-            methods.append('token')
-        project_id = token_data['access']['token'].get('tenant', {}).get('id')
-        domain_id = None
-        trust_id = token_data['access'].get('trust', {}).get('id')
-        access_token_id = None
-        federated_info = None
-        return (user_id, expires_at, audit_ids, methods, domain_id, project_id,
-                trust_id, access_token_id, federated_info)
-
-    def _extract_v3_token_data(self, token_data):
-        """Extract information from a v3 token reference."""
-        user_id = token_data['token']['user']['id']
-        expires_at = token_data['token']['expires_at']
-        audit_ids = token_data['token']['audit_ids']
-        methods = token_data['token'].get('methods')
-        domain_id = token_data['token'].get('domain', {}).get('id')
-        project_id = token_data['token'].get('project', {}).get('id')
-        trust_id = token_data['token'].get('OS-TRUST:trust', {}).get('id')
-        access_token_id = token_data['token'].get('OS-OAUTH1', {}).get(
-            'access_token_id')
-        federated_info = self._build_federated_info(token_data)
-
-        return (user_id, expires_at, audit_ids, methods, domain_id, project_id,
-                trust_id, access_token_id, federated_info)
-
     def _get_token_id(self, token_data):
         """Generate the token_id based upon the data in token_data.
 
@@ -192,27 +159,32 @@ class Provider(common.BaseProvider):
         :rtype: six.text_type
 
         """
-        # NOTE(lbragstad): Only v2.0 token responses include an 'access'
-        # attribute.
-        if token_data.get('access'):
-            (user_id, expires_at, audit_ids, methods, domain_id, project_id,
-                trust_id, access_token_id, federated_info) = (
-                    self._extract_v2_token_data(token_data))
-        else:
-            (user_id, expires_at, audit_ids, methods, domain_id, project_id,
-                trust_id, access_token_id, federated_info) = (
-                    self._extract_v3_token_data(token_data))
+        user_id = token_data['token']['user']['id']
+        expires_at = token_data['token']['expires_at']
+        audit_ids = token_data['token']['audit_ids']
+        methods = token_data['token'].get('methods')
+        system = token_data['token'].get('system', {}).get('all', None)
+        domain_id = token_data['token'].get('domain', {}).get('id')
+        project_id = token_data['token'].get('project', {}).get('id')
+        trust_id = token_data['token'].get('OS-TRUST:trust', {}).get('id')
+        access_token_id = token_data['token'].get('OS-OAUTH1', {}).get(
+            'access_token_id')
+        federated_info = self._build_federated_info(token_data)
+        app_cred_id = token_data['token'].get('application_credential',
+                                              {}).get('id')
 
         return self.token_formatter.create_token(
             user_id,
             expires_at,
             audit_ids,
             methods=methods,
+            system=system,
             domain_id=domain_id,
             project_id=project_id,
             trust_id=trust_id,
             federated_info=federated_info,
-            access_token_id=access_token_id
+            access_token_id=access_token_id,
+            app_cred_id=app_cred_id
         )
 
     @property
