@@ -14,12 +14,9 @@
 
 import uuid
 
-from oslo_utils import timeutils
-
 from keystone import assignment
 from keystone.common import controller
-from keystone.common import dependency
-from keystone.common import driver_hints
+from keystone.common import provider_api
 from keystone.common import utils
 from keystone.common import validation
 from keystone import exception
@@ -27,6 +24,9 @@ from keystone.i18n import _
 from keystone.trust import schema
 
 
+PROVIDERS = provider_api.ProviderAPIs
+
+
 def _trustor_trustee_only(trust, user_id):
     if user_id not in [trust.get('trustee_user_id'),
                        trust.get('trustor_user_id')]:
@@ -34,8 +34,6 @@ def _trustor_trustee_only(trust, user_id):
             action=_('Requested user has no relation to this trust'))
 
 
-@dependency.requires('assignment_api', 'identity_api', 'resource_api',
-                     'role_api', 'token_provider_api', 'trust_api')
 class TrustV3(controller.V3Controller):
     collection_name = "trusts"
     member_name = "trust"
@@ -49,7 +47,7 @@ class TrustV3(controller.V3Controller):
         return super(TrustV3, cls).base_url(context, path=path)
 
     def get_trust(self, request, trust_id):
-        trust = self.trust_api.get_trust(trust_id)
+        trust = PROVIDERS.trust_api.get_trust(trust_id)
         _trustor_trustee_only(trust, request.context.user_id)
         self._fill_in_roles(request.context_dict, trust)
         return TrustV3.wrap_member(request.context_dict, trust)
@@ -65,7 +63,7 @@ class TrustV3(controller.V3Controller):
             if isinstance(trust_role, dict):
                 trust_role = trust_role['id']
             try:
-                matching_role = self.role_api.get_role(trust_role)
+                matching_role = PROVIDERS.role_api.get_role(trust_role)
                 full_role = assignment.controllers.RoleV3.wrap_member(
                     context, matching_role)['role']
                 trust_full_roles.append(full_role)
@@ -79,23 +77,13 @@ class TrustV3(controller.V3Controller):
             'previous': None}
 
     def _normalize_role_list(self, trust_roles):
-        roles = [{'id': role['id']} for role in trust_roles if 'id' in role]
-        names = [role['name'] for role in trust_roles if 'id' not in role]
-        if len(names):
-            # Long way
-            for name in names:
-                hints = driver_hints.Hints()
-                hints.add_filter("name", name, case_sensitive=True)
-                found_roles = self.role_api.list_roles(hints)
-                if not found_roles:
-                    raise exception.RoleNotFound(
-                        _("Role %s is not defined") % name
-                    )
-                elif len(found_roles) == 1:
-                    roles.append({'id': found_roles[0]['id']})
-                else:
-                    raise exception.AmbiguityError(resource='role',
-                                                   name=name)
+        roles = []
+        for role in trust_roles:
+            if role.get('id'):
+                roles.append({'id': role['id']})
+            else:
+                role_api = PROVIDERS.role_api
+                roles.append(role_api.get_unique_role_by_name(role['name']))
         return roles
 
     def _find_redelegated_trust(self, request):
@@ -107,11 +95,20 @@ class TrustV3(controller.V3Controller):
                 action = _('Redelegation allowed for delegated by trust only')
                 raise exception.ForbiddenAction(action=action)
 
-            redelegated_trust = self.trust_api.get_trust(src_trust_id)
+            redelegated_trust = PROVIDERS.trust_api.get_trust(src_trust_id)
         else:
             redelegated_trust = None
         return redelegated_trust
 
+    def _check_unrestricted(self, token):
+        auth_methods = token['methods']
+        if 'application_credential' in auth_methods:
+            td = token.token_data['token']
+            if td['application_credential']['restricted']:
+                action = _("Using method 'application_credential' is not "
+                           "allowed for managing trusts.")
+                raise exception.ForbiddenAction(action=action)
+
     @controller.protected()
     def create_trust(self, request, trust):
         """Create a new trust.
@@ -120,6 +117,10 @@ class TrustV3(controller.V3Controller):
 
         """
         validation.lazy_validate(schema.trust_create, trust)
+
+        token = request.auth_context['token']
+        self._check_unrestricted(token)
+
         redelegated_trust = self._find_redelegated_trust(request)
 
         if trust.get('project_id') and not trust.get('roles'):
@@ -132,7 +133,7 @@ class TrustV3(controller.V3Controller):
             raise exception.ForbiddenAction(action=action)
 
         # ensure trustee exists
-        self.identity_api.get_user(trust['trustee_user_id'])
+        PROVIDERS.identity_api.get_user(trust['trustee_user_id'])
 
         # Normalize roles
         normalized_roles = self._normalize_role_list(trust.get('roles', []))
@@ -141,7 +142,7 @@ class TrustV3(controller.V3Controller):
         trust['expires_at'] = self._parse_expiration_date(
             trust.get('expires_at'))
         trust_id = uuid.uuid4().hex
-        new_trust = self.trust_api.create_trust(
+        new_trust = PROVIDERS.trust_api.create_trust(
             trust_id,
             trust,
             normalized_roles,
@@ -154,13 +155,13 @@ class TrustV3(controller.V3Controller):
     def _get_trustor_roles(self, trust):
         original_trust = trust.copy()
         while original_trust.get('redelegated_trust_id'):
-            original_trust = self.trust_api.get_trust(
+            original_trust = PROVIDERS.trust_api.get_trust(
                 original_trust['redelegated_trust_id'])
 
         if not self._attribute_is_empty(trust, 'project_id'):
-            self.resource_api.get_project(original_trust['project_id'])
+            PROVIDERS.resource_api.get_project(original_trust['project_id'])
             # Get a list of roles including any domain specific roles
-            assignment_list = self.assignment_api.list_role_assignments(
+            assignment_list = PROVIDERS.assignment_api.list_role_assignments(
                 user_id=original_trust['trustor_user_id'],
                 project_id=original_trust['project_id'],
                 effective=True, strip_domain_roles=False)
@@ -179,15 +180,7 @@ class TrustV3(controller.V3Controller):
     def _parse_expiration_date(self, expiration_date):
         if expiration_date is None:
             return None
-        if not expiration_date.endswith('Z'):
-            expiration_date += 'Z'
-        try:
-            expiration_time = timeutils.parse_isotime(expiration_date)
-        except ValueError:
-            raise exception.ValidationTimeStampError()
-        if timeutils.is_older_than(expiration_time, 0):
-            raise exception.ValidationExpirationError()
-        return expiration_time
+        return utils.parse_expiration_date(expiration_date)
 
     @controller.protected()
     def list_trusts(self, request):
@@ -197,20 +190,24 @@ class TrustV3(controller.V3Controller):
 
         if not request.params:
             self.assert_admin(request)
-            trusts += self.trust_api.list_trusts()
+            trusts += PROVIDERS.trust_api.list_trusts()
 
         action = _('Cannot list trusts for another user')
         if trustor_user_id:
             if trustor_user_id != request.context.user_id:
                 raise exception.Forbidden(action=action)
 
-            trusts += self.trust_api.list_trusts_for_trustor(trustor_user_id)
+            trusts += PROVIDERS.trust_api.list_trusts_for_trustor(
+                trustor_user_id
+            )
 
         if trustee_user_id:
             if trustee_user_id != request.context.user_id:
                 raise exception.ForbiddenAction(action=action)
 
-            trusts += self.trust_api.list_trusts_for_trustee(trustee_user_id)
+            trusts += PROVIDERS.trust_api.list_trusts_for_trustee(
+                trustee_user_id
+            )
 
         for trust in trusts:
             # get_trust returns roles, list_trusts does not
@@ -227,33 +224,35 @@ class TrustV3(controller.V3Controller):
 
     @controller.protected()
     def delete_trust(self, request, trust_id):
-        trust = self.trust_api.get_trust(trust_id)
+        token = request.auth_context['token']
+        self._check_unrestricted(token)
+
+        trust = PROVIDERS.trust_api.get_trust(trust_id)
 
         if (request.context.user_id != trust.get('trustor_user_id') and
                 not request.context.is_admin):
             action = _('Only admin or trustor can delete a trust')
             raise exception.ForbiddenAction(action=action)
 
-        self.trust_api.delete_trust(
+        PROVIDERS.trust_api.delete_trust(
             trust_id, initiator=request.audit_initiator
         )
 
     @controller.protected()
     def list_roles_for_trust(self, request, trust_id):
         trust = self.get_trust(request, trust_id)['trust']
-        _trustor_trustee_only(trust, request.context.user_id)
         return {'roles': trust['roles'],
                 'links': trust['roles_links']}
 
     @controller.protected()
     def get_role_for_trust(self, request, trust_id, role_id):
         """Get a role that has been assigned to a trust."""
-        trust = self.trust_api.get_trust(trust_id)
+        trust = PROVIDERS.trust_api.get_trust(trust_id)
         _trustor_trustee_only(trust, request.context.user_id)
 
         if not any(role['id'] == role_id for role in trust['roles']):
             raise exception.RoleNotFound(role_id=role_id)
 
-        role = self.role_api.get_role(role_id)
+        role = PROVIDERS.role_api.get_role(role_id)
         return assignment.controllers.RoleV3.wrap_member(request.context_dict,
                                                          role)
